@@ -4,7 +4,7 @@
  *
  *   Purpose:	implementation of classes of typed arrays for netCDF
  *
- *   $Header: /upc/share/CVS/netcdf-3/cxx/ncvalues.cc,v 1.1 1997/04/21 15:44:51 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/cxx/ncvalues.cc,v 1.4 1997/11/19 20:33:29 russ Exp $
  *********************************************************************/
 
 #include <iostream.h>		// for debugging
@@ -33,19 +33,25 @@ ostream& operator<< (ostream& os, const NcValues& vals)
 implement(NcValues,ncbyte)
 implement(NcValues,char)
 implement(NcValues,short)
+implement(NcValues,int)
 implement(NcValues,nclong)
+implement(NcValues,long)
 implement(NcValues,float)
 implement(NcValues,double)
 
 Ncbytes_for_one_implement(ncbyte)
 Ncbytes_for_one_implement(char)
 Ncbytes_for_one_implement(short)
+Ncbytes_for_one_implement(int)
 Ncbytes_for_one_implement(nclong)
+Ncbytes_for_one_implement(long)
 Ncbytes_for_one_implement(float)
 Ncbytes_for_one_implement(double)
 
 as_ncbyte_implement(short)
+as_ncbyte_implement(int)
 as_ncbyte_implement(nclong)
+as_ncbyte_implement(long)
 as_ncbyte_implement(float)
 as_ncbyte_implement(double)
 
@@ -60,7 +66,9 @@ inline ncbyte NcValues_ncbyte::as_ncbyte( long n ) const
 }
 
 as_char_implement(short)
+as_char_implement(int)
 as_char_implement(nclong)
+as_char_implement(long)
 as_char_implement(float)
 as_char_implement(double)
 
@@ -74,7 +82,9 @@ inline char NcValues_char::as_char( long n ) const
     return the_values[n];
 }
 
+as_short_implement(int)
 as_short_implement(nclong)
+as_short_implement(long)
 as_short_implement(float)
 as_short_implement(double)
 
@@ -93,6 +103,40 @@ inline short NcValues_short::as_short( long n ) const
     return the_values[n];
 }
 
+
+as_int_implement(float)
+as_int_implement(double)
+
+inline int NcValues_ncbyte::as_int( long n ) const
+{
+    return the_values[n];
+}
+
+inline int NcValues_char::as_int( long n ) const
+{
+    return the_values[n];
+}
+
+inline int NcValues_short::as_int( long n ) const
+{
+    return the_values[n];
+}
+
+inline int NcValues_int::as_int( long n ) const
+{
+    return the_values[n];
+}
+
+inline int NcValues_nclong::as_int( long n ) const
+{
+    return the_values[n];
+}
+
+inline int NcValues_long::as_int( long n ) const
+{
+    return the_values[n];
+}
+
 as_nclong_implement(float)
 as_nclong_implement(double)
 
@@ -111,11 +155,21 @@ inline nclong NcValues_short::as_nclong( long n ) const
     return the_values[n];
 }
 
+inline nclong NcValues_int::as_nclong( long n ) const
+{
+    return the_values[n];
+}
+
 inline nclong NcValues_nclong::as_nclong( long n ) const
 {
     return the_values[n];
 }
 
+inline nclong NcValues_long::as_nclong( long n ) const
+{
+    return the_values[n];
+}
+
 as_long_implement(float)
 as_long_implement(double)
 
@@ -134,27 +188,43 @@ inline long NcValues_short::as_long( long n ) const
     return the_values[n];
 }
 
+inline long NcValues_int::as_long( long n ) const
+{
+    return the_values[n];
+}
+
 inline long NcValues_nclong::as_long( long n ) const
 {
     return the_values[n];
 }
 
+inline long NcValues_long::as_long( long n ) const
+{
+    return the_values[n];
+}
+
 as_float_implement(ncbyte)
 as_float_implement(char)
 as_float_implement(short)
+as_float_implement(int)
 as_float_implement(nclong)
+as_float_implement(long)
 as_float_implement(float)
 as_float_implement(double)
 
 as_double_implement(ncbyte)
 as_double_implement(char)
 as_double_implement(short)
+as_double_implement(int)
 as_double_implement(nclong)
+as_double_implement(long)
 as_double_implement(float)
 as_double_implement(double)
 
 as_string_implement(short)
+as_string_implement(int)
 as_string_implement(nclong)
+as_string_implement(long)
 as_string_implement(float)
 as_string_implement(double)
 
@@ -183,6 +253,15 @@ ostream& NcValues_short::print(ostream& os) const
     return os;
 }
 
+ostream& NcValues_int::print(ostream& os) const
+{
+    for(int i = 0; i < the_number - 1; i++)
+      os << the_values[i] << ", ";
+    if (the_number > 0)
+      os << the_values[the_number-1] ;
+    return os;
+}
+
 ostream& NcValues_nclong::print(ostream& os) const
 {
     for(int i = 0; i < the_number - 1; i++)
@@ -192,6 +271,15 @@ ostream& NcValues_nclong::print(ostream& os) const
     return os;
 }
 
+ostream& NcValues_long::print(ostream& os) const
+{
+    for(int i = 0; i < the_number - 1; i++)
+      os << the_values[i] << ", ";
+    if (the_number > 0)
+      os << the_values[the_number-1] ;
+    return os;
+}
+
 ostream& NcValues_ncbyte::print(ostream& os) const
 {
     for(int i = 0; i < the_number - 1; i++)
