@@ -4,7 +4,7 @@
  *
  *   Purpose:	Implements class interface for netCDF over C interface
  *
- *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdf.cc,v 1.1 1997/04/21 15:44:55 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdf.cc,v 1.8 1998/01/16 19:48:26 russ Exp $
  *********************************************************************/
 
 #include <string.h>
@@ -171,6 +171,7 @@ NcVar* NcFile::add_var(NcToken name, NcType type, int ndims, const NcDim** dims)
     NcVar* varp =
       new NcVar(this, ncvardef(the_id, name, (nc_type) type, ndims, dimids));
     variables[n] = varp;
+    delete [] dimids;
     return varp;
 }
 
@@ -183,7 +184,8 @@ NcBool NcFile::add_att(NcToken aname, TYPE val)				      \
 NcFile_add_scalar_att(char)
 NcFile_add_scalar_att(ncbyte)
 NcFile_add_scalar_att(short)
-NcFile_add_scalar_att(nclong)
+NcFile_add_scalar_att(int)
+NcFile_add_scalar_att(long)
 NcFile_add_scalar_att(float)
 NcFile_add_scalar_att(double)
 NcFile_add_scalar_att(const char*)
@@ -197,7 +199,8 @@ NcBool NcFile::add_att(NcToken aname, int n, const TYPE* val)		      \
 NcFile_add_vector_att(char)
 NcFile_add_vector_att(ncbyte)
 NcFile_add_vector_att(short)
-NcFile_add_vector_att(nclong)
+NcFile_add_vector_att(int)
+NcFile_add_vector_att(long)
 NcFile_add_vector_att(float)
 NcFile_add_vector_att(double)
 
@@ -428,14 +431,18 @@ NcDim::~NcDim( void )
 }
 
 #define Nc_as(TYPE) name2(as_,TYPE)
-#define NcTypedComponent_as(TYPE)					      \
-TYPE NcTypedComponent::Nc_as(TYPE)( long n ) const			      \
-{									      \
-    return values()->Nc_as(TYPE)(n);					      \
+#define NcTypedComponent_as(TYPE)				          \
+TYPE NcTypedComponent::Nc_as(TYPE)( long n ) const		          \
+{								          \
+  NcValues *tmp = values();                                               \
+  TYPE rval = tmp->Nc_as(TYPE)(n);                                        \
+  delete tmp;                                                             \
+  return rval;                                                            \
 }
 NcTypedComponent_as(ncbyte)
 NcTypedComponent_as(char)
 NcTypedComponent_as(short)
+NcTypedComponent_as(int)
 NcTypedComponent_as(nclong)
 NcTypedComponent_as(long)
 NcTypedComponent_as(float)
@@ -443,7 +450,10 @@ NcTypedComponent_as(double)
 
 char* NcTypedComponent::as_string( long n ) const
 {
-    return values()->as_string(n);
+    NcValues *tmp = values();
+    char *rval = tmp->as_string(n);
+    delete tmp;
+    return rval;
 }
 
 NcTypedComponent::NcTypedComponent ( NcFile* nc )
@@ -462,8 +472,8 @@ NcValues* NcTypedComponent::get_space( long numVals ) const
       case ncDouble:
 	valp = new NcValues_double(numVals);
 	break;
-      case ncLong:
-	valp = new NcValues_nclong(numVals);
+      case ncInt:
+	valp = new NcValues_int(numVals);
 	break;
       case ncShort:
 	valp = new NcValues_short(numVals);
@@ -648,7 +658,8 @@ NcBool NcVar::put_rec( const TYPE* vals,                                      \
 NcVar_put_rec(ncbyte)
 NcVar_put_rec(char)
 NcVar_put_rec(short)
-NcVar_put_rec(nclong)
+NcVar_put_rec(int)
+NcVar_put_rec(long)
 NcVar_put_rec(float)
 NcVar_put_rec(double)
 
@@ -689,6 +700,7 @@ NcVar_get_index(ncbyte)
 NcVar_get_index(char)
 NcVar_get_index(short)
 NcVar_get_index(nclong)
+NcVar_get_index(long)
 NcVar_get_index(float)
 NcVar_get_index(double)
     
@@ -728,7 +740,8 @@ NcBool NcVar::put( const TYPE* vals,					      \
 NcVar_put_array(ncbyte)
 NcVar_put_array(char)
 NcVar_put_array(short)
-NcVar_put_array(nclong)
+NcVar_put_array(int)
+NcVar_put_array(long)
 NcVar_put_array(float)
 NcVar_put_array(double)
 
@@ -748,7 +761,8 @@ NcBool NcVar::put( const TYPE* vals, const long* count )		      \
 NcVar_put_nd_array(ncbyte)
 NcVar_put_nd_array(char)
 NcVar_put_nd_array(short)
-NcVar_put_nd_array(nclong)
+NcVar_put_nd_array(int)
+NcVar_put_nd_array(long)
 NcVar_put_nd_array(float)
 NcVar_put_nd_array(double)
 
@@ -787,7 +801,8 @@ NcBool NcVar::get( TYPE* vals,						      \
 NcVar_get_array(ncbyte)
 NcVar_get_array(char)
 NcVar_get_array(short)
-NcVar_get_array(nclong)
+NcVar_get_array(int)
+NcVar_get_array(long)
 NcVar_get_array(float)
 NcVar_get_array(double)
 
@@ -807,7 +822,8 @@ NcBool NcVar::get( TYPE* vals, const long* count ) const		      \
 NcVar_get_nd_array(ncbyte)
 NcVar_get_nd_array(char)
 NcVar_get_nd_array(short)
-NcVar_get_nd_array(nclong)
+NcVar_get_nd_array(int)
+NcVar_get_nd_array(long)
 NcVar_get_nd_array(float)
 NcVar_get_nd_array(double)
 
@@ -815,13 +831,14 @@ NcVar_get_nd_array(double)
 // to args provided, rest to zeros.
 NcBool NcVar::set_cur(long c0, long c1, long c2, long c3, long c4)
 {
-    long t[5];
+    long t[6];
     t[0] = c0;
     t[1] = c1;
     t[2] = c2;
     t[3] = c3;
     t[4] = c4;
-    for(int j = 0; j < 5; j++) {
+    t[5] = -1;
+    for(int j = 0; j < 6; j++) { // find how many parameters were used
 	int i;
 	if (t[j] == -1) {
 	    if (num_dims() < j)
@@ -863,7 +880,8 @@ NcBool NcVar::add_att(NcToken aname, TYPE val)				      \
 NcVar_add_scalar_att(char)
 NcVar_add_scalar_att(ncbyte)
 NcVar_add_scalar_att(short)
-NcVar_add_scalar_att(nclong)
+NcVar_add_scalar_att(int)
+NcVar_add_scalar_att(long)
 NcVar_add_scalar_att(double)
 
 NcBool NcVar::add_att(NcToken aname, float val)
@@ -900,7 +918,8 @@ NcBool NcVar::add_att(NcToken aname, int len, const TYPE* vals)		      \
 NcVar_add_vector_att(char)
 NcVar_add_vector_att(ncbyte)
 NcVar_add_vector_att(short)
-NcVar_add_vector_att(nclong)
+NcVar_add_vector_att(int)
+NcVar_add_vector_att(long)
 NcVar_add_vector_att(float)
 NcVar_add_vector_att(double)
 
