@@ -3,7 +3,7 @@
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
 
-/* $Id: fort-v2compat.c,v 1.16 1997/06/16 14:26:48 steve Exp $ */
+/* $Id: fort-v2compat.c,v 1.21 1998/02/17 20:11:47 steve Exp $ */
 
 /*
  *  Source for netCDF FORTRAN jacket library.
@@ -24,17 +24,153 @@
 
 /* LINTLIBRARY */
 
+#include	"ncconfig.h"
 #include	<ctype.h>
 #include        <string.h>
 #include	<stdlib.h>
 #include	<stdio.h>
 #include	"netcdf.h"
-#include	"ncconfig.h"
 #include	"nfconfig.inc"
 #include	"ncfortran.h"	/* netCDF FORTRAN-calling-C interface */
 #include	"fort-lib.h"
 
 
+/*
+ * Additional Fortran-calling-C interface types specific to the version 2
+ * API:
+ */
+#define NCOPTS		FINT2CINT	/* Input, netCDF options argument */
+#define PNCOPTS		PCINT2FINT	/* Output, netCDF options argument */
+#define CLOBMODE	FINT2CINT	/* Input, clobber-mode argument */
+#define PRCODE		PCINT2FINT	/* Output, return-code argument */
+#define RWMODE		FINT2CINT	/* Input, read-write mode argument */
+#define DIMLEN		FINT2CINT	/* Input, dimension-length argument */
+#define PDIMLEN		PCINT2FINT	/* Output, dimension-length argument */
+#define LENSTR		FINT2CINT	/* Input, string-length argument */
+#define ATTLEN		FINT2CINT	/* Input, attribute length argument */
+#define PATTLEN		PCINT2FINT	/* Output, attribute length argument */
+#define FILLMODE	FINT2CINT	/* Input, fill-mode argument */
+
+#define V2IMAP_cfINT(N,A,B,X,Y,Z)	DOUBLE_cfINT(N,A,B,X,Y,Z)
+#define V2IMAPVVVVVVV_cfTYPE		NF_INTEGER
+#define V2IMAP_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,V2IMAP,A,B,C,D,E)
+#define V2IMAP_cfH(S,U,B)
+#define V2IMAP_cfQ(B)			ptrdiff_t B[MAX_NC_DIMS];
+#define V2IMAP_cfT(M,I,A,B,D)		f2c_v2imap(*fncid, *fvarid-1, A, B)
+#define V2IMAP_cfR(A,B,D)
+
+
+/**
+ * Convert a Version 2 Fortran IMAP vector into a Version 3 C imap vector.
+ */
+static ptrdiff_t*
+f2c_v2imap(int ncid, int varid, const int* fimap, ptrdiff_t* cimap)
+{
+    int		status;
+    int		rank;
+    nc_type	datatype;
+
+    if ((status = nc_inq_vartype(ncid, varid, &datatype)) != NC_NOERR ||
+	(status = nc_inq_varndims(ncid, varid, &rank)) != NC_NOERR ||
+	rank <= 0)
+    {
+	return NULL;
+    }
+
+    /* else */
+    if (fimap[0] == 0)
+    {
+	/*
+	 * Special Fortran version 2 semantics: use external netCDF variable 
+	 * structure.
+	 */
+	int		dimids[NC_MAX_VAR_DIMS];
+	int		idim;
+	size_t	total;
+
+	if (nc_inq_vardimid(ncid, varid, dimids) != NC_NOERR)
+	    return NULL;
+
+	for (total = 1, idim = rank - 1; idim >= 0; --idim)
+	{
+	    size_t	length;
+
+	    cimap[idim] = total;
+
+	    if (nc_inq_dimlen(ncid, dimids[idim], &length) != NC_NOERR)
+		return NULL;
+
+	    total *= length;
+	}
+    }
+    else
+    {
+	/*
+	 * Regular Fortran version 2 semantics: convert byte counts to
+	 * element counts.
+	 */
+	int	idim;
+	size_t	size;
+
+	switch (datatype)
+	{
+
+	    case NC_CHAR:
+		size = sizeof(char);
+		break;
+	    case NC_BYTE:
+#		if NF_INT1_IS_C_SIGNED_CHAR
+		    size = sizeof(signed char);
+#		elif NF_INT1_IS_C_SHORT
+		    size = sizeof(short);
+#		elif NF_INT1_IS_C_INT
+		    size = sizeof(int);
+#		elif NF_INT1_IS_C_LONG
+		    size = sizeof(long);
+#		endif
+		break;
+	    case NC_SHORT:
+#		if NF_INT2_IS_C_SHORT
+		    size = sizeof(short);
+#		elif NF_INT2_IS_C_INT
+		    size = sizeof(int);
+#		elif NF_INT2_IS_C_LONG
+		    size = sizeof(long);
+#		endif
+		break;
+	    case NC_INT:
+#		if NF_INT_IS_C_INT
+		    size = sizeof(int);
+#		elif NF_INT_IS_C_LONG
+		    size = sizeof(long);
+#		endif
+		break;
+	    case NC_FLOAT:
+#		if NF_REAL_IS_C_FLOAT
+		    size = sizeof(float);
+#		elif NF_REAL_IS_C_DOUBLE
+		    size = sizeof(double);
+#		endif
+		break;
+	    case NC_DOUBLE:
+#		if NF_DOUBLEPRECISION_IS_C_FLOAT
+		    size = sizeof(float);
+#		elif NF_DOUBLEPRECISION_IS_C_DOUBLE
+		    size = sizeof(double);
+#		endif
+		break;
+	    default:
+		return NULL;
+	}
+
+	for (idim = 0; idim < rank; ++idim)
+	    cimap[idim] = fimap[rank - 1 - idim] / size;
+    }
+
+    return cimap;
+}
+
+
 /*
  * Compute the product of dimensional counts.
  */
@@ -78,7 +214,7 @@ c_ncpopt(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB1(c_ncpopt,NCPOPT,ncpopt,
-    INT)
+    NCOPTS)
 
 
 /*
@@ -94,7 +230,7 @@ c_ncgopt(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB1(c_ncgopt,NCGOPT,ncgopt,
-    PINT)
+    PNCOPTS)
 
 
 /*
@@ -142,8 +278,8 @@ c_nccre(
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN3(INT,c_nccre,NCCRE,nccre,
-    STRING,INT,PINT)
+FCALLSCFUN3(NF_INT,c_nccre,NCCRE,nccre,
+    STRING,CLOBMODE,PRCODE)
 
 
 /*
@@ -191,8 +327,8 @@ c_ncopn(
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN3(INT,c_ncopn,NCOPN,ncopn,
-    STRING,INT,PINT)
+FCALLSCFUN3(NF_INT,c_ncopn,NCOPN,ncopn,
+    STRING,RWMODE,PRCODE)
 
 
 /*
@@ -220,8 +356,8 @@ c_ncddef (
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN4(INT,c_ncddef,NCDDEF,ncddef,
-    INT,STRING,INT,PINT)
+FCALLSCFUN4(NF_INT,c_ncddef,NCDDEF,ncddef,
+    NCID,STRING,DIMLEN,PRCODE)
 
 
 /*
@@ -248,8 +384,8 @@ c_ncdid (
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN3(INT,c_ncdid,NCDID,ncdid,
-    INT,STRING,PINT)
+FCALLSCFUN3(NF_INT,c_ncdid,NCDID,ncdid,
+    NCID,STRING,PRCODE)
 
 
 /*
@@ -283,8 +419,8 @@ c_ncvdef (
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN6(INT,c_ncvdef,NCVDEF,ncvdef,
-    NCID,STRING,TYPE,NDIMS,DIMIDS,PINT)
+FCALLSCFUN6(NF_INT,c_ncvdef,NCVDEF,ncvdef,
+    NCID,STRING,TYPE,NDIMS,DIMIDS,PRCODE)
 
 
 /*
@@ -311,8 +447,8 @@ c_ncvid (
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN3(INT,c_ncvid,NCVID,ncvid,
-    NCID,STRING,PINT)
+FCALLSCFUN3(NF_INT,c_ncvid,NCVID,ncvid,
+    NCID,STRING,PRCODE)
 
 
 /*
@@ -320,13 +456,13 @@ FCALLSCFUN3(INT,c_ncvid,NCVID,ncvid,
  */
 static int
 c_nctlen (
-    int datatype,	/* netCDF datatype */
+    nc_type datatype,	/* netCDF datatype */
     int* rcode		/* returned error code */
 )
 {
     int itype;
 
-    *rcode = ((itype = (int) nctypelen ((nc_type) datatype)) == -1)
+    *rcode = ((itype = (int) nctypelen (datatype)) == -1)
 		?  ncerr
 		: 0;
 
@@ -334,8 +470,8 @@ c_nctlen (
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN2(INT,c_nctlen,NCTLEN,nctlen,
-    INT,PINT)
+FCALLSCFUN2(NF_INT,c_nctlen,NCTLEN,nctlen,
+    TYPE,PRCODE)
 
 
 /*
@@ -354,7 +490,7 @@ c_ncclos (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB2(c_ncclos,NCCLOS,ncclos,
-    NCID,PINT)
+    NCID,PRCODE)
 
 
 /*
@@ -373,7 +509,7 @@ c_ncredf (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB2(c_ncredf,NCREDF,ncredf,
-    NCID,PINT)
+    NCID,PRCODE)
 
 
 /*
@@ -392,7 +528,7 @@ c_ncendf (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB2(c_ncendf,NCENDF,ncendf,
-    NCID,PINT)
+    NCID,PRCODE)
 
 
 /*
@@ -415,7 +551,7 @@ c_ncinq (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB6(c_ncinq,NCINQ,ncinq,
-    NCID,PINT,PINT,PINT,PDIMID,PINT)
+    NCID,PNDIMS,PNVARS,PNATTS,PDIMID,PRCODE)
 
 
 /*
@@ -435,7 +571,7 @@ c_ncsnc(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB2(c_ncsnc,NCSNC,ncsnc,
-    NCID,PINT)
+    NCID,PRCODE)
 
 
 /*
@@ -456,7 +592,7 @@ c_ncabor (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB2(c_ncabor,NCABOR,ncabor,
-    NCID,PINT)
+    NCID,PRCODE)
 
 
 /*
@@ -484,7 +620,7 @@ c_ncdinq (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncdinq,NCDINQ,ncdinq,
-    NCID,DIMID,PSTRING,PINT,PINT)
+    NCID,DIMID,PSTRING,PDIMLEN,PRCODE)
 
 
 /*
@@ -505,7 +641,7 @@ c_ncdren (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB4(c_ncdren,NCDREN,ncdren,
-    NCID,DIMID,STRING,PINT)
+    NCID,DIMID,STRING,PRCODE)
 
 
 /*
@@ -531,7 +667,7 @@ c_ncvinq (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB8(c_ncvinq,NCVINQ,ncvinq,
-    NCID,VARID,PSTRING,PTYPE,PNDIMS,PDIMIDS,PNATTS,PINT)
+    NCID,VARID,PSTRING,PTYPE,PNDIMS,PDIMIDS,PNATTS,PRCODE)
 
 
 /*
@@ -566,6 +702,9 @@ c_ncvpt1 (
 #	    elif NF_INT1_IS_C_INT
 		status = nc_put_var1_int(ncid, varid, indices,
 					   (const int*)value);
+#	    elif NF_INT1_IS_C_LONG
+		status = nc_put_var1_long(ncid, varid, indices,
+					   (const long*)value);
 #	    endif
 	    break;
 	case NC_SHORT:
@@ -575,11 +714,19 @@ c_ncvpt1 (
 #	    elif NF_INT2_IS_C_INT
 		status = nc_put_var1_int(ncid, varid, indices,
 					   (const int*)value);
+#	    elif NF_INT2_IS_C_LONG
+		status = nc_put_var1_long(ncid, varid, indices,
+					   (const long*)value);
 #	    endif
 	    break;
 	case NC_INT:
-	    status = nc_put_var1_int(ncid, varid, indices,
-				       (const int*)value);
+#	    if NF_INT_IS_C_INT
+		status = nc_put_var1_int(ncid, varid, indices,
+					   (const int*)value);
+#	    elif NF_INT_IS_C_LONG
+		status = nc_put_var1_long(ncid, varid, indices,
+					   (const long*)value);
+#	    endif
 	    break;
 	case NC_FLOAT:
 #	    if NF_REAL_IS_C_FLOAT
@@ -613,7 +760,7 @@ c_ncvpt1 (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncvpt1,NCVPT1,ncvpt1,
-    NCID,VARID,COORDS,PVOID,PINT)
+    NCID,VARID,COORDS,PVOID,PRCODE)
 
 
 /* 
@@ -649,7 +796,7 @@ c_ncvp1c(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncvp1c,NCVP1C,ncvp1c,
-    NCID,VARID,COORDS,CBUF,PINT)
+    NCID,VARID,COORDS,CBUF,PRCODE)
 
 
 /*
@@ -686,6 +833,9 @@ c_ncvpt (
 #	    elif NF_INT1_IS_C_INT
 		status = nc_put_vara_int(ncid, varid, start, count,
 					   (const int*)value);
+#	    elif NF_INT1_IS_C_LONG
+		status = nc_put_vara_long(ncid, varid, start, count,
+					   (const long*)value);
 #	    endif
 	    break;
 	case NC_SHORT:
@@ -695,11 +845,19 @@ c_ncvpt (
 #	    elif NF_INT2_IS_C_INT
 		status = nc_put_vara_int(ncid, varid, start, count,
 					   (const int*)value);
+#	    elif NF_INT2_IS_C_LONG
+		status = nc_put_vara_long(ncid, varid, start, count,
+					   (const long*)value);
 #	    endif
 	    break;
 	case NC_INT:
-	    status = nc_put_vara_int(ncid, varid, start, count,
-				       (const int*)value);
+#	    if NF_INT_IS_C_INT
+		status = nc_put_vara_int(ncid, varid, start, count,
+					   (const int*)value);
+#	    elif NF_INT_IS_C_LONG
+		status = nc_put_vara_long(ncid, varid, start, count,
+					   (const long*)value);
+#	    endif
 	    break;
 	case NC_FLOAT:
 #	    if NF_REAL_IS_C_FLOAT
@@ -734,7 +892,7 @@ c_ncvpt (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB6(c_ncvpt,NCVPT,ncvpt,
-    NCID,VARID,COORDS,COUNTS,PVOID,PINT)
+    NCID,VARID,COORDS,COUNTS,PVOID,PRCODE)
 
 
 /*
@@ -785,7 +943,7 @@ c_ncvptc(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB7(c_ncvptc,NCVPTC,ncvptc,
-    NCID,VARID,COORDS,COUNTS,CBUF,INT,PINT)
+    NCID,VARID,COORDS,COUNTS,CBUF,LENSTR,PRCODE)
 
 
 /*
@@ -799,7 +957,7 @@ c_ncvptg (
     const size_t*	start,	/* multidimensional index of hypercube corner */
     const size_t*	count,	/* multidimensional hypercube edge lengths */
     const ptrdiff_t*	strides,/* netCDF variable access strides */
-    const ptrdiff_t*	basis,	/* memory values access basis vector */
+    const ptrdiff_t*	imap,	/* memory values access mapping vector */
     const void*		value,	/* block of data values to be written */
     int*		rcode	/* returned error code */
 )
@@ -813,71 +971,73 @@ c_ncvptg (
     {
 	switch (datatype)
 	{
-	    ptrdiff_t	maps[MAX_NC_DIMS];
-
 	case NC_CHAR:
 	    status = NC_ECHAR;
 	    break;
 	case NC_BYTE:
 #	    if NF_INT1_IS_C_SIGNED_CHAR
-		maps_divide(basis, rank, sizeof(signed char), maps);
 		status = nc_put_varm_schar(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const signed char*)value);
 #	    elif NF_INT1_IS_C_SHORT
-		maps_divide(basis, rank, sizeof(short), maps);
 		status = nc_put_varm_short(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const short*)value);
 #	    elif NF_INT1_IS_C_INT
-		maps_divide(basis, rank, sizeof(int), maps);
 		status = nc_put_varm_int(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const int*)value);
+#	    elif NF_INT1_IS_C_LONG
+		status = nc_put_varm_long(ncid, varid, start, count,
+					   strides, imap,
+					   (const long*)value);
 #	    endif
 	    break;
 	case NC_SHORT:
 #	    if NF_INT2_IS_C_SHORT
-		maps_divide(basis, rank, sizeof(short), maps);
 		status = nc_put_varm_short(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const short*)value);
 #	    elif NF_INT2_IS_C_INT
-		maps_divide(basis, rank, sizeof(int), maps);
 		status = nc_put_varm_int(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const int*)value);
+#	    elif NF_INT2_IS_C_LONG
+		status = nc_put_varm_long(ncid, varid, start, count,
+					   strides, imap,
+					   (const long*)value);
 #	    endif
 	    break;
 	case NC_INT:
-	    maps_divide(basis, rank, sizeof(int), maps);
-	    status = nc_put_varm_int(ncid, varid, start, count,
-				       strides, maps,
-				       (const int*)value);
+#	    if NF_INT_IS_C_INT
+		status = nc_put_varm_int(ncid, varid, start, count,
+					   strides, imap,
+					   (const int*)value);
+#	    elif NF_INT_IS_C_LONG
+		status = nc_put_varm_long(ncid, varid, start, count,
+					   strides, imap,
+					   (const long*)value);
+#	    endif
 	    break;
 	case NC_FLOAT:
 #	    if NF_REAL_IS_C_FLOAT
-		maps_divide(basis, rank, sizeof(short), maps);
 		status = nc_put_varm_float(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const float*)value);
 #	    elif NF_REAL_IS_C_DOUBLE
-		maps_divide(basis, rank, sizeof(double), maps);
 		status = nc_put_varm_double(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const double*)value);
 #	    endif
 	    break;
 	case NC_DOUBLE:
 #	    if NF_DOUBLEPRECISION_IS_C_FLOAT
-		maps_divide(basis, rank, sizeof(float), maps);
 		status = nc_put_varm_float(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const float*)value);
 #	    elif NF_DOUBLEPRECISION_IS_C_DOUBLE
-		maps_divide(basis, rank, sizeof(double), maps);
 		status = nc_put_varm_double(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (const double*)value);
 #	    endif
 	    break;
@@ -896,7 +1056,7 @@ c_ncvptg (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB8(c_ncvptg,NCVPTG,ncvptg,
-    NCID,VARID,COORDS,COUNTS,STRIDES,MAPS,PVOID,PINT)
+    NCID,VARID,COORDS,COUNTS,STRIDES,V2IMAP,PVOID,PRCODE)
 
 
 /*
@@ -910,7 +1070,7 @@ c_ncvpgc(
     const size_t*	start,	/* multidimensional index of hypercube corner */
     const size_t*	count,	/* multidimensional hypercube edge lengths */
     const ptrdiff_t*	strides,/* netCDF variable access strides */
-    const ptrdiff_t*	basis,	/* memory values access basis vector */
+    const ptrdiff_t*	imap,	/* memory values access mapping vector */
     const char*		value,	/* block of data values to be written */
     int*		rcode	/* returned error code */
 )
@@ -924,12 +1084,9 @@ c_ncvpgc(
     {
 	switch (datatype)
 	{
-	    ptrdiff_t	maps[MAX_NC_DIMS];
-
 	case NC_CHAR:
-	    maps_divide(basis, rank, sizeof(char), maps);
 	    status = nc_put_varm_text(ncid, varid, start, count,
-				       strides, maps,
+				       strides, imap,
 				       value);
 	    break;
 	default:
@@ -950,7 +1107,7 @@ c_ncvpgc(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB8(c_ncvpgc,NCVPGC,ncvpgc,
-    NCID,VARID,COORDS,COUNTS,STRIDES,MAPS,CBUF,PINT)
+    NCID,VARID,COORDS,COUNTS,STRIDES,V2IMAP,CBUF,PRCODE)
 
 
 /*
@@ -985,6 +1142,9 @@ c_ncvgt1 (
 #	    elif NF_INT1_IS_C_INT
 		status = nc_get_var1_int(ncid, varid, indices,
 					   (int*)value);
+#	    elif NF_INT1_IS_C_LONG
+		status = nc_get_var1_long(ncid, varid, indices,
+					   (long*)value);
 #	    endif
 	    break;
 	case NC_SHORT:
@@ -994,11 +1154,19 @@ c_ncvgt1 (
 #	    elif NF_INT2_IS_C_INT
 		status = nc_get_var1_int(ncid, varid, indices,
 					   (int*)value);
+#	    elif NF_INT2_IS_C_LONG
+		status = nc_get_var1_long(ncid, varid, indices,
+					   (long*)value);
 #	    endif
 	    break;
 	case NC_INT:
-	    status = nc_get_var1_int(ncid, varid, indices,
-				       (int*)value);
+#	    if NF_INT_IS_C_INT
+		status = nc_get_var1_int(ncid, varid, indices,
+					   (int*)value);
+#	    elif NF_INT_IS_C_LONG
+		status = nc_get_var1_long(ncid, varid, indices,
+					   (long*)value);
+#	    endif
 	    break;
 	case NC_FLOAT:
 #	    if NF_REAL_IS_C_FLOAT
@@ -1033,7 +1201,7 @@ c_ncvgt1 (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncvgt1,NCVGT1,ncvgt1,
-    NCID,VARID,COORDS,PVOID,PINT)
+    NCID,VARID,COORDS,PVOID,PRCODE)
 
 
 /*
@@ -1077,7 +1245,7 @@ c_ncvg1c(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncvg1c,NCVG1C,ncvg1c,
-    NCID,VARID,COORDS,CBUF,PINT)
+    NCID,VARID,COORDS,CBUF,PRCODE)
 
 
 /*
@@ -1114,6 +1282,9 @@ c_ncvgt(
 #	    elif NF_INT1_IS_C_INT
 		status = nc_get_vara_int(ncid, varid, start, count,
 					   (int*)value);
+#	    elif NF_INT1_IS_C_LONG
+		status = nc_get_vara_long(ncid, varid, start, count,
+					   (long*)value);
 #	    endif
 	    break;
 	case NC_SHORT:
@@ -1123,11 +1294,19 @@ c_ncvgt(
 #	    elif NF_INT2_IS_C_INT
 		status = nc_get_vara_int(ncid, varid, start, count,
 					   (int*)value);
+#	    elif NF_INT2_IS_C_LONG
+		status = nc_get_vara_long(ncid, varid, start, count,
+					   (long*)value);
 #	    endif
 	    break;
 	case NC_INT:
-	    status = nc_get_vara_int(ncid, varid, start, count,
-				       (int*)value);
+#	    if NF_INT_IS_C_INT
+		status = nc_get_vara_int(ncid, varid, start, count,
+					   (int*)value);
+#	    elif NF_INT_IS_C_LONG
+		status = nc_get_vara_long(ncid, varid, start, count,
+					   (long*)value);
+#	    endif
 	    break;
 	case NC_FLOAT:
 #	    if NF_REAL_IS_C_FLOAT
@@ -1162,7 +1341,7 @@ c_ncvgt(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB6(c_ncvgt,NCVGT,ncvgt,
-    NCID,VARID,COORDS,COUNTS,PVOID,PINT)
+    NCID,VARID,COORDS,COUNTS,PVOID,PRCODE)
 
 
 /*
@@ -1211,7 +1390,7 @@ c_ncvgtc(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB7(c_ncvgtc,NCVGTC,ncvgtc,
-    NCID,VARID,COORDS,COUNTS,CBUF,INT,PINT)
+    NCID,VARID,COORDS,COUNTS,CBUF,LENSTR,PRCODE)
 
 
 /*
@@ -1225,7 +1404,7 @@ c_ncvgtg (
     const size_t*	start,	/* multidimensional index of hypercube corner */
     const size_t*	count,	/* multidimensional hypercube edge lengths */
     const ptrdiff_t*	strides,/* netCDF variable access strides */
-    const ptrdiff_t*	basis,	/* memory values access basis vector */
+    const ptrdiff_t*	imap,	/* memory values access basis vector */
     void*		value,	/* block of data values to be read */
     int*		rcode	/* returned error code */
 )
@@ -1239,71 +1418,73 @@ c_ncvgtg (
     {
 	switch (datatype)
 	{
-	    ptrdiff_t	maps[MAX_NC_DIMS];
-
 	case NC_CHAR:
 	    status = NC_ECHAR;
 	    break;
 	case NC_BYTE:
 #	    if NF_INT1_IS_C_SIGNED_CHAR
-		maps_divide(basis, rank, sizeof(signed char), maps);
 		status = nc_get_varm_schar(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (signed char*)value);
 #	    elif NF_INT1_IS_C_SHORT
-		maps_divide(basis, rank, sizeof(short), maps);
 		status = nc_get_varm_short(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (short*)value);
 #	    elif NF_INT1_IS_C_INT
-		maps_divide(basis, rank, sizeof(int), maps);
 		status = nc_get_varm_int(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (int*)value);
+#	    elif NF_INT1_IS_C_LONG
+		status = nc_get_varm_long(ncid, varid, start, count,
+					   strides, imap,
+					   (long*)value);
 #	    endif
 	    break;
 	case NC_SHORT:
 #	    if NF_INT2_IS_C_SHORT
-		maps_divide(basis, rank, sizeof(short), maps);
 		status = nc_get_varm_short(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (short*)value);
 #	    elif NF_INT2_IS_C_INT
-		maps_divide(basis, rank, sizeof(int), maps);
 		status = nc_get_varm_int(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (int*)value);
+#	    elif NF_INT2_IS_C_LONG
+		status = nc_get_varm_long(ncid, varid, start, count,
+					   strides, imap,
+					   (long*)value);
 #	    endif
 	    break;
 	case NC_INT:
-	    maps_divide(basis, rank, sizeof(int), maps);
-	    status = nc_get_varm_int(ncid, varid, start, count,
-				       strides, maps,
-				       (int*)value);
+#	    if NF_INT_IS_C_INT
+		status = nc_get_varm_int(ncid, varid, start, count,
+					   strides, imap,
+					   (int*)value);
+#	    elif NF_INT_IS_C_LONG
+		status = nc_get_varm_long(ncid, varid, start, count,
+					   strides, imap,
+					   (long*)value);
+#	    endif
 	    break;
 	case NC_FLOAT:
 #	    if NF_REAL_IS_C_FLOAT
-		maps_divide(basis, rank, sizeof(short), maps);
 		status = nc_get_varm_float(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (float*)value);
 #	    elif NF_REAL_IS_C_DOUBLE
-		maps_divide(basis, rank, sizeof(double), maps);
 		status = nc_get_varm_double(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (double*)value);
 #	    endif
 	    break;
 	case NC_DOUBLE:
 #	    if NF_DOUBLEPRECISION_IS_C_FLOAT
-		maps_divide(basis, rank, sizeof(float), maps);
 		status = nc_get_varm_float(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (float*)value);
 #	    elif NF_DOUBLEPRECISION_IS_C_DOUBLE
-		maps_divide(basis, rank, sizeof(double), maps);
 		status = nc_get_varm_double(ncid, varid, start, count,
-					   strides, maps,
+					   strides, imap,
 					   (double*)value);
 #	    endif
 	    break;
@@ -1321,7 +1502,7 @@ c_ncvgtg (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB8(c_ncvgtg,NCVGTG,ncvgtg,
-    NCID,VARID,COORDS,COUNTS,STRIDES,MAPS,PVOID,PINT)
+    NCID,VARID,COORDS,COUNTS,STRIDES,V2IMAP,PVOID,PRCODE)
 
 
 /*
@@ -1335,7 +1516,7 @@ c_ncvggc(
     const size_t*	start,	/* multidimensional index of hypercube corner */
     const size_t*	count,	/* multidimensional hypercube edge lengths */
     const ptrdiff_t*	strides,/* netCDF variable access strides */
-    const ptrdiff_t*	basis,	/* memory values access basis vector */
+    const ptrdiff_t*	imap,	/* memory values access basis vector */
     char*		value,	/* block of data values to be written */
     int*		rcode	/* returned error code */
 )
@@ -1349,12 +1530,9 @@ c_ncvggc(
     {
 	switch (datatype)
 	{
-	    ptrdiff_t	maps[MAX_NC_DIMS];
-
 	case NC_CHAR:
-	    maps_divide(basis, rank, sizeof(char), maps);
 	    status = nc_get_varm_text(ncid, varid, start, count,
-				       strides, maps,
+				       strides, imap,
 				       value);
 	    break;
 	default:
@@ -1374,7 +1552,7 @@ c_ncvggc(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB8(c_ncvggc,NCVGGC,ncvggc,
-    NCID,VARID,COORDS,COUNTS,STRIDES,MAPS,CBUF,PINT)
+    NCID,VARID,COORDS,COUNTS,STRIDES,V2IMAP,CBUF,PRCODE)
 
 
 /*
@@ -1395,7 +1573,7 @@ c_ncvren (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB4(c_ncvren,NCVREN,ncvren,
-    NCID,VARID,STRING,PINT)
+    NCID,VARID,STRING,PRCODE)
 
 
 /*
@@ -1430,6 +1608,9 @@ c_ncapt (
 #	elif NF_INT1_IS_C_INT
 	    status = nc_put_att_int(ncid, varid, attname, datatype,
 				       attlen, (const int*)value);
+#	elif NF_INT1_IS_C_LONG
+	    status = nc_put_att_long(ncid, varid, attname, datatype,
+				       attlen, (const long*)value);
 #	endif
 	break;
     case NC_SHORT:
@@ -1439,11 +1620,19 @@ c_ncapt (
 #	elif NF_INT2_IS_C_INT
 	    status = nc_put_att_int(ncid, varid, attname, datatype,
 				       attlen, (const int*)value);
+#	elif NF_INT2_IS_C_LONG
+	    status = nc_put_att_long(ncid, varid, attname, datatype,
+				       attlen, (const long*)value);
 #	endif
 	break;
     case NC_INT:
-	status = nc_put_att_int(ncid, varid, attname, datatype,
-				   attlen, (const int*)value);
+#	if NF_INT_IS_C_INT
+	    status = nc_put_att_int(ncid, varid, attname, datatype,
+				       attlen, (const int*)value);
+#	elif NF_INT_IS_C_LONG
+	    status = nc_put_att_long(ncid, varid, attname, datatype,
+				       attlen, (const long*)value);
+#	endif
 	break;
     case NC_FLOAT:
 #	if NF_REAL_IS_C_FLOAT
@@ -1476,7 +1665,7 @@ c_ncapt (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB7(c_ncapt,NCAPT,ncapt,
-    NCID,VARID,STRING,TYPE,COUNT,PVOID,PINT)
+    NCID,VARID,STRING,TYPE,COUNT,PVOID,PRCODE)
 
 
 /*
@@ -1511,7 +1700,7 @@ c_ncaptc(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB7(c_ncaptc,NCAPTC,ncaptc,
-    NCID,VARID,STRING,TYPE,COUNT,CBUF,PINT)
+    NCID,VARID,STRING,TYPE,COUNT,CBUF,PRCODE)
 
 
 /*
@@ -1536,7 +1725,7 @@ c_ncainq (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB6(c_ncainq,NCAINQ,ncainq,
-    NCID,VARID,STRING,PTYPE,PINT,PINT)
+    NCID,VARID,STRING,PTYPE,PATTLEN,PRCODE)
 
 
 /*
@@ -1571,6 +1760,9 @@ c_ncagt(
 #    	elif NF_INT1_IS_C_INT
 		status = nc_get_att_int(ncid, varid, attname, 
 					   (int*)value);
+#    	elif NF_INT1_IS_C_LONG
+		status = nc_get_att_long(ncid, varid, attname, 
+					   (long*)value);
 #    	endif
 	    break;
 	case NC_SHORT:
@@ -1580,11 +1772,19 @@ c_ncagt(
 #    	elif NF_INT2_IS_C_INT
 		status = nc_get_att_int(ncid, varid, attname, 
 					   (int*)value);
+#    	elif NF_INT2_IS_C_LONG
+		status = nc_get_att_long(ncid, varid, attname, 
+					   (long*)value);
 #    	endif
 	    break;
 	case NC_INT:
-	    status = nc_get_att_int(ncid, varid, attname, 
-				       (int*)value);
+#	    if NF_INT_IS_C_INT
+		status = nc_get_att_int(ncid, varid, attname, 
+					   (int*)value);
+#	    elif NF_INT_IS_C_LONG
+		status = nc_get_att_long(ncid, varid, attname, 
+					   (long*)value);
+#	    endif
 	    break;
 	case NC_FLOAT:
 #    	if NF_REAL_IS_C_FLOAT
@@ -1618,7 +1818,7 @@ c_ncagt(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncagt,NCAGT,ncagt,
-    NCID,VARID,STRING,PVOID,PINT)
+    NCID,VARID,STRING,PVOID,PRCODE)
 
 
 /*
@@ -1631,7 +1831,7 @@ c_ncagtc(
     int		varid,		/* variable ID */
     const char*	attname,	/* attribute name */
     char*	value,		/* pointer to data values */
-    size_t	attlen,		/* length of string argument */
+    int		attlen,		/* length of string argument */
     int*	rcode		/* returned error code */
 )
 {
@@ -1673,7 +1873,7 @@ c_ncagtc(
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB6(c_ncagtc,NCAGTC,ncagtc,
-    NCID,VARID,STRING,CBUF,COUNT,PINT)
+    NCID,VARID,STRING,CBUF,ATTLEN,PRCODE)
 
 
 /*
@@ -1697,7 +1897,7 @@ c_ncacpy (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB6(c_ncacpy,NCACPY,ncacpy,
-    NCID1,VARID1,STRING,NCID2,VARID2,PINT)
+    NCID1,VARID1,STRING,NCID2,VARID2,PRCODE)
 
 
 /*
@@ -1720,7 +1920,7 @@ c_ncanam (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncanam,NCANAM,ncanam,
-    NCID,VARID,ATTID,PSTRING,PINT)
+    NCID,VARID,ATTID,PSTRING,PRCODE)
 
 
 /*
@@ -1742,7 +1942,7 @@ c_ncaren (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB5(c_ncaren,NCAREN,ncaren,
-    NCID,VARID,STRING,STRING,PINT)
+    NCID,VARID,STRING,STRING,PRCODE)
 
 
 /*
@@ -1763,7 +1963,7 @@ c_ncadel (
 
 /* FORTRAN interface to the above. */
 FCALLSCSUB4(c_ncadel,NCADEL,ncadel,
-    NCID,VARID,STRING,PINT)
+    NCID,VARID,STRING,PRCODE)
 
 
 /*
@@ -1786,5 +1986,5 @@ c_ncsfil (
 }
 
 /* FORTRAN interface to the above. */
-FCALLSCFUN3(INT,c_ncsfil,NCSFIL,ncsfil,
-    NCID,INT,PINT)
+FCALLSCFUN3(NF_INT,c_ncsfil,NCSFIL,ncsfil,
+    NCID,FILLMODE,PRCODE)
