@@ -1,4 +1,4 @@
-/* $Id: ncfortran.h,v 1.9 1997/06/03 22:25:28 steve Exp $ */
+/* $Id: ncfortran.h,v 1.14 1998/02/25 19:24:24 steve Exp $ */
 
 /*
  * This file contains `cfortran.h' definitions that are specific to the
@@ -11,6 +11,87 @@
 
 
 #include "cfortran.h"
+#include "nfconfig.inc"
+
+
+/*
+ * The type of a Fortran INTEGER:
+ */
+#if NF_INT_IS_C_INT
+#   define	NF_INTEGER	int
+#   define	NF_INT		INT
+#elif NF_INT_IS_C_LONG
+#   define	NF_INTEGER	long
+#   define	NF_INT		LONG
+#else
+#   include "NF_INT_IS_C_... not defined"
+#endif
+
+/*
+ * Input, Fortran INTEGER converted to C "int" (helper macro):
+ */
+#define FINT2CINT_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
+#define FINT2CINT_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
+#define FINT2CINT_cfN(  T,A)		const NF_INTEGER *A
+#define FINT2CINT_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
+#define FINT2CINT_cfT(M,I,A,B,D)	(int)*A
+
+
+/*
+ * Input, Fortran INTEGER converted to C "size_t":
+ */
+#define FINT2CSIZET_cfINT(N,A,B,X,Y,Z)		SIMPLE_cfINT(N,A,B,X,Y,Z)
+#define FINT2CSIZET_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
+#define FINT2CSIZET_cfN(  T,A)			const NF_INTEGER *A
+#define FINT2CSIZET_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
+#define FINT2CSIZET_cfT(M,I,A,B,D)		(size_t)*A
+
+
+/*
+ * Input/Output, I/O size hint:
+ */
+#define PCHUNKSIZEHINT_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
+#define PCHUNKSIZEHINT_cfTYPE		NF_INTEGER
+#define PCHUNKSIZEHINT_cfSTR(N,T,A,B,C,D,E)	\
+					_(CFARGS,N)(T,PCHUNKSIZEHINT,A,B,C,D,E)
+#define PCHUNKSIZEHINT_cfH(S,U,B)
+#define PCHUNKSIZEHINT_cfQ(B)		size_t B;
+#define PCHUNKSIZEHINT_cfT(M,I,A,B,D)	((void*)(B=*A), &B)
+#define PCHUNKSIZEHINT_cfR(A,B,D)	*A=B;
+
+
+/*
+ * Output, C "int" converted to Fortran INTEGER (helper macro):
+ */
+#define PCINT2FINT_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
+#define PCINT2FINT_cfTYPE		NF_INTEGER
+#define PCINT2FINT_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PCINT2FINT,A,B,C,D,E)
+#define PCINT2FINT_cfH(S,U,B)
+#define PCINT2FINT_cfQ(B)		int B;
+#define PCINT2FINT_cfT(M,I,A,B,D)	&B
+#define PCINT2FINT_cfR(A,B,D)		*A=B;
+
+
+/*
+ * Input, Fortran index converted to C index:
+ */
+#define FNDX2CNDX_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
+#define FNDX2CNDX_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
+#define FNDX2CNDX_cfN(  T,A)		const NF_INTEGER *A
+#define FNDX2CNDX_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
+#define FNDX2CNDX_cfT(M,I,A,B,D)	(*A-1)
+
+
+/*
+ * Output, C index converted to Fortran index:
+ */
+#define PCNDX2FNDX_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
+#define PCNDX2FNDX_cfTYPE		NF_INTEGER
+#define PCNDX2FNDX_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PCNDX2FNDX,A,B,C,D,E)
+#define PCNDX2FNDX_cfH(S,U,B)
+#define PCNDX2FNDX_cfQ(B)		int B;
+#define PCNDX2FNDX_cfT(M,I,A,B,D)	&B
+#define PCNDX2FNDX_cfR(A,B,D)		*A=B+1;
 
 
 /*******************************************************************************
@@ -56,23 +137,13 @@
 /*
  * Input, attribute ID:
  */
-#define ATTID_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
-#define ATTID_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define ATTID_cfN(  T,A)		int *A
-#define ATTID_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
-#define ATTID_cfT(M,I,A,B,D)		(*A-1)
+#define ATTID	FNDX2CNDX
 
 
 /*
  * Output, attribute ID:
  */
-#define PATTID_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PATTID_cfTYPE			int
-#define PATTID_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PATTID,A,B,C,D,E)
-#define PATTID_cfH(S,U,B)
-#define PATTID_cfQ(B)
-#define PATTID_cfT(M,I,A,B,D)		A
-#define PATTID_cfR(A,B,D)		++*A;
+#define PATTID	PCNDX2FNDX
 
 
 /*******************************************************************************
@@ -85,7 +156,7 @@
  */
 #define TYPE_cfINT(N,A,B,X,Y,Z)		SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define TYPE_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define TYPE_cfN(  T,A)			int *A
+#define TYPE_cfN(  T,A)			NF_INTEGER *A
 #define TYPE_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define TYPE_cfT(M,I,A,B,D)		((nc_type)*A)
 
@@ -94,7 +165,7 @@
  * Output, netCDF type:
  */
 #define PTYPE_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PTYPE_cfTYPE			int
+#define PTYPE_cfTYPE			NF_INTEGER
 #define PTYPE_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PTYPE,A,B,C,D,E)
 #define PTYPE_cfH(S,U,B)
 #define PTYPE_cfQ(B)			nc_type B;
@@ -112,39 +183,39 @@
  */
 #define NDIMS_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define NDIMS_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define NDIMS_cfN(  T,A)		int *fndims
+#define NDIMS_cfN(  T,A)		NF_INTEGER *fndims
 #define NDIMS_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
-#define NDIMS_cfT(M,I,A,B,D)		*fndims
+#define NDIMS_cfT(M,I,A,B,D)		((int)*fndims)
 
 
 /*
  * Output number-of-dimensions:
  */
-#define	PNDIMS				PINT
+#define	PNDIMS	PCINT2FINT
 
 
 /*
  * Input number-of-variables:
  */
-#define NVARS				INT
+#define NVARS	FINT2CINT
 
 
 /*
  * Output number-of-variables:
  */
-#define	PNVARS				PINT
+#define	PNVARS	PNDIMS
 
 
 /*
  * Input number-of-attributes:
  */
-#define NATTS				INT
+#define NATTS	FINT2CINT
 
 
 /*
  * Output number-of-attributes:
  */
-#define	PNATTS				PINT
+#define	PNATTS	PNDIMS
 
 
 /*******************************************************************************
@@ -187,6 +258,8 @@
 #   define INT1VAR_cfN(  T,A)		const short *A
 #elif NF_INT1_IS_C_INT
 #   define INT1VAR_cfN(  T,A)		const int *A
+#elif NF_INT1_IS_C_LONG
+#   define INT1VAR_cfN(  T,A)		const long *A
 #endif
 #define INT1VAR_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define INT1VAR_cfT(M,I,A,B,D)		A
@@ -203,6 +276,8 @@
 #   define PINT1VAR_cfN(  T,A)		short *A
 #elif NF_INT1_IS_C_INT
 #   define PINT1VAR_cfN(  T,A)		int *A
+#elif NF_INT1_IS_C_LONG
+#   define PINT1VAR_cfN(  T,A)		long *A
 #endif
 #define PINT1VAR_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define PINT1VAR_cfT(M,I,A,B,D)		A
@@ -229,6 +304,8 @@
 #   define INT2VAR_cfN(  T,A)		const short *A
 #elif NF_INT2_IS_C_INT
 #   define INT2VAR_cfN(  T,A)		const int *A
+#elif NF_INT2_IS_C_LONG
+#   define INT2VAR_cfN(  T,A)		const long *A
 #endif
 #define INT2VAR_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define INT2VAR_cfT(M,I,A,B,D)		A
@@ -243,6 +320,8 @@
 #   define PINT2VAR_cfN(  T,A)		short *A
 #elif NF_INT2_IS_C_INT
 #   define PINT2VAR_cfN(  T,A)		int *A
+#elif NF_INT2_IS_C_LONG
+#   define PINT2VAR_cfN(  T,A)		long *A
 #endif
 #define PINT2VAR_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define PINT2VAR_cfT(M,I,A,B,D)		A
@@ -265,7 +344,7 @@
  */
 #define INTVAR_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define INTVAR_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define INTVAR_cfN(  T,A)		const int *A
+#define INTVAR_cfN(  T,A)		const NF_INTEGER *A
 #define INTVAR_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define INTVAR_cfT(M,I,A,B,D)		A
 
@@ -275,7 +354,7 @@
  */
 #define PINTVAR_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define PINTVAR_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define PINTVAR_cfN(  T,A)		int *A
+#define PINTVAR_cfN(  T,A)		NF_INTEGER *A
 #define PINTVAR_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define PINTVAR_cfT(M,I,A,B,D)		A
 
@@ -373,53 +452,37 @@
  * Output, `size_t' variable:
  */
 #define PSIZET_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PSIZET_cfTYPE			int *
+#define PSIZET_cfTYPE			NF_INTEGER
 #define PSIZET_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PSIZET,A,B,C,D,E)
 #define PSIZET_cfH(S,U,B)
 #define PSIZET_cfQ(B)			size_t B;
-#define PSIZET_cfT(M,I,A,B,D)		(B=*A,&B)
+#define PSIZET_cfT(M,I,A,B,D)		&B
 #define PSIZET_cfR(A,B,D)		*A=B;
 
 
-/*
- * Output, origin-1 index:
- */
-#define PINDEX_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PINDEX_cfTYPE			int *
-#define PINDEX_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PINDEX,A,B,C,D,E)
-#define PINDEX_cfH(S,U,B)
-#define PINDEX_cfQ(B)			int B;
-#define PINDEX_cfT(M,I,A,B,D)		(B=*A-1,&B)
-#define PINDEX_cfR(A,B,D)		*A=B+1;
-
-
 /*
  * Input dimension-ID:
  */
-#define DIMID_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
-#define DIMID_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define DIMID_cfN(  T,A)		int *A
-#define DIMID_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
-#define DIMID_cfT(M,I,A,B,D)		(*A-1)
+#define DIMID	FNDX2CNDX
 
 
 /*
  * Output, dimension-ID:
  */
 #define PDIMID_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PDIMID_cfTYPE			int
+#define PDIMID_cfTYPE			NF_INTEGER
 #define PDIMID_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PDIMID,A,B,C,D,E)
 #define PDIMID_cfH(S,U,B)
-#define PDIMID_cfQ(B)
-#define PDIMID_cfT(M,I,A,B,D)		A
-#define PDIMID_cfR(A,B,D)		++*A;
+#define PDIMID_cfQ(B)			int B;
+#define PDIMID_cfT(M,I,A,B,D)		&B
+#define PDIMID_cfR(A,B,D)		*A=(B == -1 ? -1 : B+1);
 
 
 /*
  * Input, dimension-ID vector:
  */
 #define DIMIDS_cfINT(N,A,B,X,Y,Z)	DOUBLE_cfINT(N,A,B,X,Y,Z)
-#define DIMIDSVVVVVVV_cfTYPE		int
+#define DIMIDSVVVVVVV_cfTYPE		NF_INTEGER
 #define DIMIDS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,DIMIDS,A,B,C,D,E)
 #define DIMIDS_cfH(S,U,B)
 #define DIMIDS_cfQ(B)			int B[MAX_NC_DIMS];
@@ -431,7 +494,7 @@
  * Output, dimension-ID vector:
  */
 #define PDIMIDS_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PDIMIDS_cfTYPE			int
+#define PDIMIDS_cfTYPE			NF_INTEGER
 #define PDIMIDS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PDIMIDS,A,B,C,D,E)
 #define PDIMIDS_cfH(S,U,B)
 #define PDIMIDS_cfQ(B)			int B[MAX_NC_DIMS];
@@ -444,7 +507,7 @@
  */
 #define NCID_cfINT(N,A,B,X,Y,Z)		SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define NCID_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define NCID_cfN(  T,A)			int *fncid
+#define NCID_cfN(  T,A)			NF_INTEGER *fncid
 #define NCID_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define NCID_cfT(M,I,A,B,D)		*fncid
 
@@ -452,18 +515,14 @@
 /*
  * Two, input, netCDF dataset IDs:
  */
-#define NCID1				INT
-#define NCID2				INT
+#define NCID1	FINT2CINT
+#define NCID2	FINT2CINT
 
 
 /*
  * Output, netCDF dataset ID:
  */
-#define PNCID_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
-#define PNCID_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define PNCID_cfN(  T,A)		int *fncid
-#define PNCID_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
-#define PNCID_cfT(M,I,A,B,D)		fncid
+#define PNCID	PCINT2FINT
 
 
 /*
@@ -471,7 +530,7 @@
  */
 #define VARID_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define VARID_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define VARID_cfN(  T,A)		int *fvarid
+#define VARID_cfN(  T,A)		NF_INTEGER *fvarid
 #define VARID_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define VARID_cfT(M,I,A,B,D)		(*fvarid-1)
 
@@ -481,13 +540,13 @@
  */
 #define VARID1_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define VARID1_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define VARID1_cfN(  T,A)		int *fvarid1
+#define VARID1_cfN(  T,A)		NF_INTEGER *fvarid1
 #define VARID1_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define VARID1_cfT(M,I,A,B,D)		(*fvarid1-1)
 
 #define VARID2_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define VARID2_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define VARID2_cfN(  T,A)		int *fvarid2
+#define VARID2_cfN(  T,A)		NF_INTEGER *fvarid2
 #define VARID2_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
 #define VARID2_cfT(M,I,A,B,D)		(*fvarid2-1)
 
@@ -495,20 +554,14 @@
 /*
  * Output, netCDF variable ID:
  */
-#define PVARID_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PVARID_cfTYPE			int
-#define PVARID_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PVARID,A,B,C,D,E)
-#define PVARID_cfH(S,U,B)
-#define PVARID_cfQ(B)
-#define PVARID_cfT(M,I,A,B,D)		A
-#define PVARID_cfR(A,B,D)		++*A;
+#define PVARID	PCNDX2FNDX
 
 
 /*
  * Input, co-ordinate vector:
  */
 #define COORDS_cfINT(N,A,B,X,Y,Z)	DOUBLE_cfINT(N,A,B,X,Y,Z)
-#define COORDSVVVVVVV_cfTYPE		int
+#define COORDSVVVVVVV_cfTYPE		NF_INTEGER
 #define COORDS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,COORDS,A,B,C,D,E)
 #define COORDS_cfH(S,U,B)
 #define COORDS_cfQ(B)			size_t B[MAX_NC_DIMS];
@@ -521,28 +574,22 @@
  */
 #define COUNT_cfINT(N,A,B,X,Y,Z)	SIMPLE_cfINT(N,A,B,X,Y,Z)
 #define COUNT_cfSEP(T,  B)		SIMPLE_cfSEP(T,B)
-#define COUNT_cfN(  T,A)		int *A
+#define COUNT_cfN(  T,A)		const NF_INTEGER *A
 #define COUNT_cfSTR(N,T,A,B,C,D,E)	SIMPLE_cfSTR(N,T,A,B,C,D,E)
-#define COUNT_cfT(M,I,A,B,D)		*A
+#define COUNT_cfT(M,I,A,B,D)		(size_t)*A
 
 
 /*
  * Output count:
  */
-#define PCOUNT_cfINT(N,A,B,X,Y,Z)	PDOUBLE_cfINT(N,A,B,X,Y,Z)
-#define PCOUNT_cfTYPE			int
-#define PCOUNT_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PCOUNT,A,B,C,D,E)
-#define PCOUNT_cfH(S,U,B)
-#define PCOUNT_cfQ(B)			size_t B;
-#define PCOUNT_cfT(M,I,A,B,D)		&B
-#define PCOUNT_cfR(A,B,D)		*A=B;
+#define PCOUNT	PSIZET
 
 
 /*
  * Input, count vector:
  */
 #define COUNTS_cfINT(N,A,B,X,Y,Z)	DOUBLE_cfINT(N,A,B,X,Y,Z)
-#define COUNTSVVVVVVV_cfTYPE		int
+#define COUNTSVVVVVVV_cfTYPE		NF_INTEGER
 #define COUNTS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,COUNTS,A,B,C,D,E)
 #define COUNTS_cfH(S,U,B)
 #define COUNTS_cfQ(B)			size_t B[MAX_NC_DIMS];
@@ -554,7 +601,7 @@
  * Input, stride vector:
  */
 #define STRIDES_cfINT(N,A,B,X,Y,Z)	DOUBLE_cfINT(N,A,B,X,Y,Z)
-#define STRIDESVVVVVVV_cfTYPE		int
+#define STRIDESVVVVVVV_cfTYPE		NF_INTEGER
 #define STRIDES_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,STRIDES,A,B,C,D,E)
 #define STRIDES_cfH(S,U,B)
 #define STRIDES_cfQ(B)			ptrdiff_t B[MAX_NC_DIMS];
@@ -566,7 +613,7 @@
  * Input, mapping vector:
  */
 #define MAPS_cfINT(N,A,B,X,Y,Z)		DOUBLE_cfINT(N,A,B,X,Y,Z)
-#define MAPSVVVVVVV_cfTYPE		int
+#define MAPSVVVVVVV_cfTYPE		NF_INTEGER
 #define MAPS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,MAPS,A,B,C,D,E)
 #define MAPS_cfH(S,U,B)
 #define MAPS_cfQ(B)			ptrdiff_t B[MAX_NC_DIMS];
@@ -581,9 +628,8 @@
 #if defined(f2cFortran)
 
 /*
- *
  * The f2c(1) utility on BSD/OS and Linux systems adds an additional
- * underscore suffix (besides the standard one) to global names that have
+ * underscore suffix (besides the usual one) to global names that have
  * an embedded underscore.  For example, `nfclose' becomes `nfclose_',
  * but `nf_close' becomes `nf_close__.  Consequently, we have to modify
  * some names.
