@@ -116,7 +116,7 @@ nc_strerror(int err)
 	case NC_EMAXATTS:
 	    return "NC_MAX_ATTRS exceeded";
 	case NC_EBADTYPE:
-	    return "Not a netCDF data type";
+	    return "Not a netCDF data type or _FillValue type mismatch";
 	case NC_EBADDIM:
 	    return "Invalid dimension id or name";
 	case NC_EUNLIMPOS:
