@@ -2,9 +2,10 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: nc.c,v 2.106 1997/06/10 04:05:26 davis Exp $ */
+/* $Id: nc.c,v 2.113 1997/12/12 00:12:44 davis Exp $ */
 
 #include "nc.h"
+#include "rnd.h"
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
@@ -77,8 +78,6 @@ free_NC(NC *ncp)
 {
 	if(ncp == NULL)
 		return;
-	if(ncp->scratch != NULL)
-		free(ncp->scratch);
 	free_NC_dimarrayV(&ncp->dims);
 	free_NC_attrarrayV(&ncp->attrs);
 	free_NC_vararrayV(&ncp->vars);
@@ -86,12 +85,8 @@ free_NC(NC *ncp)
 }
 
 
-#ifndef NC_SIZEHINT
-#define NC_SIZEHINT NCIO_SIZEHINT_DEFAULT
-#endif
-
 static NC *
-new_NC(void)
+new_NC(const size_t *chunkp)
 {
 	NC *ncp;
 
@@ -102,8 +97,8 @@ new_NC(void)
 
 	ncp->xsz = MIN_NC_XSZ;
 	assert(ncp->xsz == ncx_len_NC(ncp));
-
-	ncp->chunk = NC_SIZEHINT;
+	
+	ncp->chunk = chunkp != NULL ? *chunkp : NC_SIZEHINT_DEFAULT;
 
 	return ncp;
 }
@@ -131,7 +126,6 @@ dup_NC(const NC *ref)
 	ncp->begin_rec = ref->begin_rec;
 	ncp->recsize = ref->recsize;
 	ncp->numrecs = ref->numrecs;
-	ncp->scratch = NULL; /* N.B. */
 	return ncp;
 err:
 	free_NC(ncp);
@@ -185,26 +179,39 @@ ncx_howmany(nc_type type, size_t xbufsize)
 	return(0);
 }
 
+#define	D_RNDUP(x, align) _RNDUP(x, (off_t)(align))
 
 /*
  * Compute each variable's 'begin' offset,
  * update 'begin_rec' as well.
  */
 static void
-NC_begins(NC *ncp)
+NC_begins(NC *ncp,
+	size_t h_minfree, size_t v_align,
+	size_t v_minfree, size_t r_align)
 {
 	size_t ii;
 	off_t index = 0;
 	NC_var **vpp;
 	NC_var *last = NULL;
 
+	if(v_align == NC_ALIGN_CHUNK)
+		v_align = ncp->chunk;
+	if(r_align == NC_ALIGN_CHUNK)
+		r_align = ncp->chunk;
+
 	ncp->xsz = ncx_len_NC(ncp);
 
 	if(ncp->vars.nelems == 0) 
 		return;
 
 	index = (off_t) ncp->xsz;
-	ncp->begin_var = index;
+	ncp->begin_var = D_RNDUP(index, v_align);
+	if(ncp->begin_var - index < h_minfree)
+	{
+		ncp->begin_var = D_RNDUP(index + (off_t)h_minfree, v_align);
+	}
+	index = ncp->begin_var;
 
 	/* loop thru vars, first pass is for the 'non-record' vars */
 	vpp = ncp->vars.value;
@@ -215,12 +222,20 @@ NC_begins(NC *ncp)
 			/* skip record variables on this pass */
 			continue;
 		}
-
+#if 0
+fprintf(stderr, "    VAR %d %s: %ld\n", ii, (*vpp)->name->cp, (long)index);
+#endif
 		(*vpp)->begin = index;
 		index += (*vpp)->len;
 	}
 
-	ncp->begin_rec = index;
+	ncp->begin_rec = D_RNDUP(index, r_align);
+	if(ncp->begin_rec - index < v_minfree)
+	{
+		ncp->begin_rec = D_RNDUP(index + (off_t)v_minfree, r_align);
+	}
+	index = ncp->begin_rec;
+
 	ncp->recsize = 0;
 
 	/* loop thru vars, second pass is for the 'record' vars */
@@ -233,6 +248,9 @@ NC_begins(NC *ncp)
 			continue;
 		}
 
+#if 0
+fprintf(stderr, "    REC %d %s: %ld\n", ii, (*vpp)->name->cp, (long)index);
+#endif
 		(*vpp)->begin = index;
 		index += (*vpp)->len;
 		ncp->recsize += (*vpp)->len;
@@ -263,10 +281,6 @@ read_numrecs(NC *ncp)
 
 	assert(!NC_indef(ncp));
 
-	status = ncp->nciop->sync(ncp->nciop);
-	if(status != NC_NOERR)
-		return status;
-
 #define NC_NUMRECS_OFFSET 4
 #define NC_NUMRECS_EXTENT 4
 	status = ncp->nciop->get(ncp->nciop,
@@ -279,6 +293,8 @@ read_numrecs(NC *ncp)
 
 	(void) ncp->nciop->rel(ncp->nciop, NC_NUMRECS_OFFSET, 0);
 
+	if(status == NC_NOERR)
+		fClr(ncp->flags, NC_NDIRTY);
 
 	return status;
 }
@@ -306,8 +322,6 @@ write_numrecs(NC *ncp)
 
 	(void) ncp->nciop->rel(ncp->nciop, NC_NUMRECS_OFFSET, RGN_MODIFIED);
 
-	status = ncp->nciop->sync(ncp->nciop);
-
 	if(status == NC_NOERR)
 		fClr(ncp->flags, NC_NDIRTY);
 
@@ -324,10 +338,6 @@ read_NC(NC *ncp)
 {
 	int status = NC_NOERR;
 
-	status = ncp->nciop->sync(ncp->nciop);
-	if(status)
-		return status;
-
 	free_NC_dimarrayV(&ncp->dims);
 	free_NC_attrarrayV(&ncp->attrs);
 	free_NC_vararrayV(&ncp->vars);
@@ -348,19 +358,10 @@ static int
 write_NC(NC *ncp)
 {
 	int status = NC_NOERR;
-	void *xp;
 
 	assert(!NC_readonly(ncp));
 
-	status = ncp->nciop->get(ncp->nciop, 0, ncp->xsz, RGN_WRITE, &xp);
-	if(status)
-		return status;
-
-	status = ncx_put_NC(&xp, ncp);
-
-	(void) ncp->nciop->rel(ncp->nciop, 0, RGN_MODIFIED);
-
-	status = ncp->nciop->sync(ncp->nciop);
+	status = ncx_put_NC(ncp, NULL, 0, 0);
 
 	if(status == NC_NOERR)
 		fClr(ncp->flags, NC_NDIRTY | NC_HDIRTY);
@@ -425,6 +426,65 @@ fillerup(NC *ncp)
 
 /* Begin endef */
 
+/*
+ */
+static int
+fill_added_recs(NC *gnu, NC *old)
+{
+	NC_var ** const gnu_varpp = (NC_var **)gnu->vars.value;
+
+	int recno = 0;
+	for(; recno < old->numrecs; recno++)
+	{
+		int varid = (int)old->vars.nelems;
+		for(; varid < (int)gnu->vars.nelems; varid++)
+		{
+			const NC_var *const gnu_varp = *(gnu_varpp + varid);
+			if(!IS_RECVAR(gnu_varp))
+			{
+				/* skip non-record variables */
+				continue;
+			}
+			/* else */
+			{
+			const int status = fill_NC_var(gnu, gnu_varp, recno);
+			if(status != NC_NOERR)
+				return status;
+			}
+		}
+	}
+
+	return NC_NOERR;
+}
+
+/*
+ */
+static int
+fill_added(NC *gnu, NC *old)
+{
+	NC_var ** const gnu_varpp = (NC_var **)gnu->vars.value;
+	int varid = (int)old->vars.nelems;
+
+	for(; varid < (int)gnu->vars.nelems; varid++)
+	{
+		const NC_var *const gnu_varp = *(gnu_varpp + varid);
+		if(IS_RECVAR(gnu_varp))
+		{
+			/* skip record variables */
+			continue;
+		}
+		/* else */
+		{
+		const int status = fill_NC_var(gnu, gnu_varp, 0);
+		if(status != NC_NOERR)
+			return status;
+		}
+	}
+
+	return NC_NOERR;
+}
+
+
 /*
  * Move the records "out". 
  * Fill as needed.
@@ -442,9 +502,11 @@ move_recs_r(NC *gnu, NC *old)
 	off_t gnu_off;
 	off_t old_off;
 	
+	/* Don't parallelize this loop */
 	for(recno = (int)old->numrecs -1; recno >= 0; recno--)
 	{
-	for(varid = (int)gnu->vars.nelems -1; varid >= 0; varid--)
+	/* Don't parallelize this loop */
+	for(varid = (int)old->vars.nelems -1; varid >= 0; varid--)
 	{
 		gnu_varp = *(gnu_varpp + varid);
 		if(!IS_RECVAR(gnu_varp))
@@ -454,18 +516,6 @@ move_recs_r(NC *gnu, NC *old)
 		}
 		/* else */
 
-		if((size_t) varid >= (int) old->vars.nelems)
-		{
-			/* a new variable */
-			if(NC_dofill(gnu))
-			{
-				status = fill_NC_var(gnu, gnu_varp, recno);
-				if(status != NC_NOERR)
-					return status;
-			}
-			continue;
-		}
-
 		/* else, a pre-existing variable */
 		old_varp = *(old_varpp + varid);
 		gnu_off = gnu_varp->begin + (off_t)(gnu->recsize * recno);
@@ -507,7 +557,9 @@ move_vars_r(NC *gnu, NC *old)
 	off_t gnu_off;
 	off_t old_off;
 	
-	for(varid = (int)gnu->vars.nelems -1; varid >= 0; varid--)
+	/* Don't parallelize this loop */
+	for(varid = (int)old->vars.nelems -1;
+		 varid >= 0; varid--)
 	{
 		gnu_varp = *(gnu_varpp + varid);
 		if(IS_RECVAR(gnu_varp))
@@ -517,19 +569,6 @@ move_vars_r(NC *gnu, NC *old)
 		}
 		/* else */
 
-		if((size_t) varid >= old->vars.nelems)
-		{
-			/* a new variable */
-			if(NC_dofill(gnu))
-			{
-				status = fill_NC_var(gnu, gnu_varp, 0);
-				if(status != NC_NOERR)
-					return status;
-			}
-			continue;
-		}
-
-		/* else, a pre-existing variable */
 		old_varp = *(old_varpp + varid);
 		gnu_off = gnu_varp->begin;
 		old_off = old_varp->begin;
@@ -568,7 +607,7 @@ move_vars_f(NC *gnu, NC *old)
 	off_t old_off;
 	
 
-	for(varid = 0; (size_t) varid < gnu->vars.nelems; varid++)
+	for(varid = 0; (size_t) varid < old->vars.nelems; varid++)
 	{
 		gnu_varp = *(gnu_varpp + varid);
 		if(IS_RECVAR(gnu_varp))
@@ -578,33 +617,20 @@ move_vars_f(NC *gnu, NC *old)
 		}
 		/* else */
 
-		if((size_t) varid < old->vars.nelems)
-		{
-			/* else, a pre-existing variable */
-			old_varp = *(old_varpp + varid);
-			gnu_off = gnu_varp->begin;
-			old_off = old_varp->begin;
-		
-			if(gnu_off == old_off)
-				continue; 	/* nothing to do */
+		old_varp = *(old_varpp + varid);
+		gnu_off = gnu_varp->begin;
+		old_off = old_varp->begin;
 	
-			assert(gnu_off < old_off);
+		if(gnu_off == old_off)
+			continue; 	/* nothing to do */
 
-			status = gnu->nciop->move(gnu->nciop, gnu_off, old_off,
-				 old_varp->len, 0);
+		assert(gnu_off < old_off);
 	
-			if(status != NC_NOERR)
-				return status;
-				continue;
-		}
+		status = gnu->nciop->move(gnu->nciop, gnu_off, old_off,
+			 old_varp->len, 0);
 
-		/* else, a new variable */
-		if(NC_dofill(gnu))
-		{
-			status = fill_NC_var(gnu, gnu_varp, 0);
-			if(status != NC_NOERR)
-				return status;
-		}
+		if(status != NC_NOERR)
+			return status;
 	}
 
 	return NC_NOERR;
@@ -612,7 +638,7 @@ move_vars_f(NC *gnu, NC *old)
 
 
 /*
- * Move the records "int". 
+ * Move the records "in". 
  * Fill as needed.
  */
 static int
@@ -685,14 +711,16 @@ move_recs_f(NC *gnu, NC *old)
  *  Common code for ncendef, ncclose(endef)
  */
 static int
-NC_endef(NC *ncp)
+NC_endef(NC *ncp,
+	size_t h_minfree, size_t v_align,
+	size_t v_minfree, size_t r_align)
 {
 	int status = NC_NOERR;
 
 	assert(!NC_readonly(ncp));
 	assert(NC_indef(ncp));
 
-	NC_begins(ncp);
+	NC_begins(ncp, h_minfree, v_align, v_minfree, r_align);
 
 	if(ncp->old != NULL)
 	{
@@ -700,16 +728,6 @@ NC_endef(NC *ncp)
 		assert(!NC_IsNew(ncp));
 		assert(fIsSet(ncp->flags, NC_INDEF));
 
-		if(ncp->dims.nelems > ncp->old->dims.nelems)
-		{
-			/* need bigger scratch space */
-			if(ncp->scratch != NULL)
-			{
-				free(ncp->scratch);
-				ncp->scratch = NULL;
-			}
-		}
-
 		if(ncp->vars.nelems != 0)
 		{
 		if(ncp->begin_rec > ncp->old->begin_rec)
@@ -772,33 +790,37 @@ NC_endef(NC *ncp)
 			/* NOOP */
 		}
 		}
-		free_NC(ncp->old);
-		ncp->old = NULL;
-		fClr(ncp->flags, NC_INDEF);
 	}
 
-	if(ncp->scratch == NULL && ncp->dims.nelems != 0)
-	{
-		ncp->scratch = (size_t *)malloc(2 *
-				 ncp->dims.nelems * sizeof(size_t));
-		if(ncp->scratch == NULL)
-			return NC_ENOMEM;
-	}
-
-	assert(pIf(ncp->dims.nelems == 0, ncp->scratch == NULL));
-
 	status = write_NC(ncp);
 	if(status != NC_NOERR)
 		return status;
 
-	if(NC_IsNew(ncp) && NC_dofill(ncp))
+	if(NC_dofill(ncp))
 	{
-		status = fillerup(ncp);
-		if(status != NC_NOERR)
-			return status;
-		
+		if(NC_IsNew(ncp))
+		{
+			status = fillerup(ncp);
+			if(status != NC_NOERR)
+				return status;
+			
+		}
+		else if(ncp->vars.nelems > ncp->old->vars.nelems)
+		{
+			status = fill_added(ncp, ncp->old);
+			if(status != NC_NOERR)
+				return status;
+			status = fill_added_recs(ncp, ncp->old);
+			if(status != NC_NOERR)
+				return status;
+		}
 	}
 
+	if(ncp->old != NULL)
+	{
+		free_NC(ncp->old);
+		ncp->old = NULL;
+	}
 
 	fClr(ncp->flags, NC_CREAT | NC_INDEF);
 	return(status);
@@ -808,7 +830,8 @@ NC_endef(NC *ncp)
 /* Public */
 
 int
-nc_create(const char * path, int ioflags, int *ncid_ptr)
+nc__create(const char * path, int ioflags, size_t initialsz,
+	size_t *chunksizehintp, int *ncid_ptr)
 {
 	NC *ncp;
 	int status;
@@ -818,14 +841,14 @@ nc_create(const char * path, int ioflags, int *ncid_ptr)
 	fSet(ioflags, NC_SHARE);
 #endif
 
-	ncp = new_NC();
+	ncp = new_NC(chunksizehintp);
 	if(ncp == NULL)
 		return NC_ENOMEM;
 
 	assert(ncp->xsz == ncx_len_NC(ncp));
 	
 	status = ncio_create(path, ioflags,
-		0,
+		initialsz,
 		0, ncp->xsz, &ncp->chunk,
 		&ncp->nciop, &xp);
 	if(status != NC_NOERR)
@@ -851,13 +874,14 @@ nc_create(const char * path, int ioflags, int *ncid_ptr)
 		fSet(ncp->flags, NC_NSYNC);
 	}
 
-	status = ncx_put_NC(&xp, ncp);
+	status = ncx_put_NC(ncp, &xp, 0, ncp->xsz);
 	if(status != NC_NOERR)
 		goto unwind_ioc;
-	(void) ncp->nciop->rel(ncp->nciop, 0, RGN_MODIFIED);
 
 	add_to_NCList(ncp);
 
+	if(chunksizehintp != NULL)
+		*chunksizehintp = ncp->chunk;
 	*ncid_ptr = ncp->nciop->fd;
 	return NC_NOERR;
 
@@ -870,9 +894,15 @@ unwind_alloc:
 	return status;
 }
 
+int
+nc_create(const char * path, int ioflags, int *ncid_ptr)
+{
+	return nc__create(path, ioflags, 0, NULL, ncid_ptr);
+}
+
 
 int
-nc_open(const char * path, int ioflags, int *ncid_ptr)
+nc__open(const char * path, int ioflags, size_t *chunksizehintp, int *ncid_ptr)
 {
 	NC *ncp;
 	int status;
@@ -881,7 +911,7 @@ nc_open(const char * path, int ioflags, int *ncid_ptr)
 	fSet(ioflags, NC_SHARE);
 #endif
 
-	ncp = new_NC();
+	ncp = new_NC(chunksizehintp);
 	if(ncp == NULL)
 		return NC_ENOMEM;
 	
@@ -909,27 +939,10 @@ nc_open(const char * path, int ioflags, int *ncid_ptr)
 	if(status != NC_NOERR)
 		goto unwind_ioc;
 
-	assert(ncp->scratch == NULL);
-	if(ncp->dims.nelems != 0)
-	{
-		ncp->scratch = (size_t *)malloc(2 *
-				 ncp->dims.nelems * sizeof(size_t));
-		if(ncp->scratch == NULL)
-		{
-			status = NC_ENOMEM;
-			goto unwind_ioc;
-		}
-
-	}
-#ifndef NDEBUG
-	else
-	{
-		assert(ncp->scratch == NULL);
-	}
-#endif
-
 	add_to_NCList(ncp);
 
+	if(chunksizehintp != NULL)
+		*chunksizehintp = ncp->chunk;
 	*ncid_ptr = ncp->nciop->fd;
 	return NC_NOERR;
 
@@ -942,6 +955,30 @@ unwind_alloc:
 	return status;
 }
 
+int
+nc_open(const char * path, int ioflags, int *ncid_ptr)
+{
+	return nc__open(path, ioflags, NULL, ncid_ptr);
+}
+
+
+int
+nc__enddef(int ncid,
+	size_t h_minfree, size_t v_align,
+	size_t v_minfree, size_t r_align)
+{
+	int status;
+	NC *ncp;
+
+	status = NC_check_id(ncid, &ncp); 
+	if(status != NC_NOERR)
+		return status;
+
+	if(!NC_indef(ncp))
+		return(NC_ENOTINDEFINE);
+
+	return(NC_endef(ncp, h_minfree, v_align, v_minfree, r_align));
+}
 
 int
 nc_enddef(int ncid)
@@ -956,7 +993,8 @@ nc_enddef(int ncid)
 	if(!NC_indef(ncp))
 		return(NC_ENOTINDEFINE);
 
-	return(NC_endef(ncp));
+	/* return(NC_endef(ncp, 0, 4096, 0, 4096)); */
+	return(NC_endef(ncp, 0, 1, 0, 1));
 }
 
 
@@ -972,7 +1010,7 @@ nc_close(int ncid)
 
 	if(NC_indef(ncp))
 	{
-		status = NC_endef(ncp);
+		status = NC_endef(ncp, 0, 1, 0, 1); /* TODO: defaults */
 		if(status != NC_NOERR )
 		{
 			(void) nc_abort(ncid);
@@ -1000,8 +1038,9 @@ nc_delete(const char * path)
 {
 	NC *ncp;
 	int status;
+	size_t chunk = 512;
 
-	ncp = new_NC();
+	ncp = new_NC(&chunk);
 	if(ncp == NULL)
 		return NC_ENOMEM;
 	
@@ -1095,10 +1134,14 @@ nc_redef(int ncid)
 	if(NC_indef(ncp))
 		return NC_EINDEFINE;
 
-	/* Do a sync. (Backward) */
-	status = NC_sync(ncp);
-	if(status != NC_NOERR)
-		return status;
+	
+	if(fIsSet(ncp->nciop->ioflags, NC_SHARE))
+	{
+		/* read in from disk */
+		status = NC_sync(ncp);
+		if(status != NC_NOERR)
+			return status;
+	}
 
 	ncp->old = dup_NC(ncp);
 	if(ncp->old == NULL)
