@@ -3,7 +3,7 @@
  *	See netcdf/COPYRIGHT file for copying and redistribution conditions.
  * 	
  */
-/* $Id: ncx_cray.c,v 1.51 1997/06/04 18:17:17 davis Exp $ */
+/* $Id: ncx_cray.c,v 1.56 1998/03/06 01:32:58 davis Exp $ */
 #ifndef _CRAY
 #error "ncx_cray.c is a cray specific implementation"
 #endif
@@ -67,12 +67,10 @@ static const int Cray2_F64 = 8;	/* CRAY float to IEEE double */
 #define FLOAT_USE_IEG 1
 #define DOUBLE_USE_IEG 1
 
-#endif /* USE_IEG */
-
-
+#if _CRAY1
 /*
  * Return the number of bits "into" a word that (void *) is.
- * N.B. This is based on the very wierd YMP address structure,
+ * N.B. This is based on the CRAY1 (PVP) address structure,
  * which puts the address within a word in the leftmost 3 bits
  * of the address.
  */
@@ -82,10 +80,17 @@ bitoff(const void *vp)
 	const size_t bitoffset = ((size_t)vp >> (64 - 6)) & 0x3f;
 	return bitoffset;
 }
+# else
+#error "Don't use IEG2CRAY, CRAY2IEG except on CRAY1 (MPP) platforms"
+#define bitoff(vp) ((size_t)(vp) % 64) /* N.B. Assumes 64 bit word */
+# endif /* _CRAY1 */
 
+#endif /* USE_IEG */
+
+#if _CRAY1
 /*
  * Return the number of bytes "into" a word that (void *) is.
- * N.B. This is based on the very wierd YMP address structure,
+ * N.B. This is based on the CRAY1 (PVP) address structure,
  * which puts the address within a word in the leftmost 3 bits
  * of the address.
  */
@@ -95,12 +100,12 @@ byteoff(const void *vp)
 	const size_t byteoffset = ((size_t)vp >> (64 - 3)) & 0x7;
 	return byteoffset;
 }
+#else
+#define byteoff(vp) ((size_t)(vp) % 8) /* N.B. Assumes 64 bit word */
+#endif /* _CRAY1 */
 
 /*
  * Return the number of bytes until the next "word" boundary
- * N.B. This is based on the very wierd YMP address structure,
- * which puts the address within a word in the leftmost 3 bits
- * of the address.
  */
 static size_t
 word_align(const void *vp)
@@ -131,7 +136,7 @@ typedef short ix_short;
 #define IX_SHORT_MAX SHORT_MAX
 
 static void
-cget_short_short(const void *xp, short *ip, short which)
+cget_short_short(const void *xp, short *ip, int which)
 {
 	const long *wp = xp;
 
@@ -159,9 +164,9 @@ cget_short_short(const void *xp, short *ip, short which)
 #define get_ix_short(xp, ip) cget_short_short((xp), (ip), byteoff(xp)/X_SIZEOF_SHORT)
 
 static int
-cput_short_short(void *xp, const short *ip, short which)
+cput_short_short(void *xp, const short *ip, int which)
 {
-	long *wp = xp;
+	word *wp = xp;
 
 	switch(which) {
 	case 0:
@@ -186,7 +191,6 @@ cput_short_short(void *xp, const short *ip, short which)
 		return NC_ERANGE;
 	return ENOERR;
 }
-#define put_ix_short(xp, ip) (void)cput_short_short((xp), (ip), byteoff(xp)/X_SIZEOF_SHORT)
 
 int
 ncx_get_short_schar(const void *xp, schar *ip)
@@ -297,7 +301,7 @@ ncx_put_short_long(void *xp, const long *ip)
 int
 ncx_put_short_float(void *xp, const float *ip)
 {
-	ix_short xx = *ip;
+	ix_short xx = (ix_short)*ip;
 	const int status = cput_short_short(xp, &xx, byteoff(xp)/X_SIZEOF_SHORT);
 	if(status != ENOERR)
 		return status;
@@ -309,7 +313,7 @@ ncx_put_short_float(void *xp, const float *ip)
 int
 ncx_put_short_double(void *xp, const double *ip)
 {
-	ix_short xx = *ip;
+	ix_short xx = (ix_short)*ip;
 	const int status = cput_short_short(xp, &xx, byteoff(xp)/X_SIZEOF_SHORT);
 	if(status != ENOERR)
 		return status;
@@ -349,7 +353,7 @@ cget_int_int(const void *xp, int *ip, int which)
 static int
 cput_int_int(void *xp, const int *ip, int which)
 {
-	long *wp = xp;
+	word *wp = xp;
 
 	if(which == 0)
 	{
@@ -987,10 +991,11 @@ get_ix_double(const void *xp, double *ip)
 	(void) memcpy(ip, xp, sizeof(double));
 }
 
-static void
+static int
 put_ix_double(void *xp, const double *ip)
 {
 	(void) memcpy(xp, ip, X_SIZEOF_DOUBLE);
+	return ENOERR;
 }
 
 #else
@@ -1844,86 +1849,114 @@ ncx_getn_short_uchar(const void **xpp, size_t nelems, uchar *tp)
 int
 ncx_getn_short_short(const void **xpp, size_t nelems, short *tp)
 {
-	size_t noff = byteoff(*xpp);
-	int status = ENOERR;
-	int ierr;
-
-	if(noff != 0)
+	if(nelems > 0)
 	{
-		for(; nelems != 0 && noff % sizeof(word) != 0; 
-			nelems--, noff += X_SIZEOF_SHORT, tp++)
-		{
-			const int bitoff = noff * 8;
-			ierr = IEG2CRAY(&Cray2_I16, &nelems, (word *)(*xpp),
-				&bitoff, tp, &UnitStride);
-			assert(ierr >= 0);
+		const int bo = bitoff(*xpp);
+		const word *wp = *xpp;
+		int ierr;
+		*xpp = ((const char *) (*xpp) + nelems * X_SIZEOF_SHORT);
+		ierr = IEG2CRAY(&Cray2_I16, &nelems, wp,
+				&bo, tp, &UnitStride);
+		assert(ierr >= 0);
 		if(ierr > 0)
-			status = NC_ERANGE;
-		}
+			return NC_ERANGE;
 	}
-
-	if(nelems == 0)
-		return status;
-		
-	assert(byteoff(*xpp) == 0);
-
-	ierr = IEG2CRAY(&Cray2_I16, &nelems, (word *)(*xpp),
-			&Zero, tp, &UnitStride);
-	assert(ierr >= 0);
-
-	*xpp = (void *)((char *)(*xpp) + nelems * X_SIZEOF_SHORT);
-
-	if(ierr > 0)
-		status = NC_ERANGE;
-	return status;
+	return ENOERR;
 }
 #else
 int
-ncx_getn_short_short(const void **xpp, size_t nelems, short *tp)
+ncx_getn_short_short(const void **xpp, const size_t nelems, short *tp)
 {
-	int which = byteoff(*xpp)/X_SIZEOF_SHORT;
-
-	if(nelems == 0)
-		return ENOERR;
-
-	if(which != 0)
+	if(nelems > 0)
 	{
-		while(which++ < 4) 
+	const word *wp = *xpp;
+	const short *const last = &tp[nelems -1];
+	const int rem = word_align(*xpp)/X_SIZEOF_SHORT;
+	*xpp = ((const char *) (*xpp) + nelems * X_SIZEOF_SHORT);
+
+	switch(rem) {
+	case 3:
+		*tp = (short)((*wp >> 32) & 0xffff);
+		if(*tp & 0x8000)
+			*tp |= (~(0xffff));
+		if(tp == last)
+			return ENOERR;
+		tp++;
+		/*FALLTHRU*/	
+	case 2:
+		*tp = (short)((*wp >> 16) & 0xffff);
+		if(*tp & 0x8000)
+			*tp |= (~(0xffff));
+		if(tp == last)
+			return ENOERR;
+		tp++;
+		/*FALLTHRU*/	
+	case 1:
+		*tp = (short)(*wp & 0xffff);
+		if(*tp & 0x8000)
+			*tp |= (~(0xffff));
+		if(tp == last)
+			return ENOERR;
+		tp++;
+		wp++; /* Note Bene */
+		/*FALLTHRU*/	
+	}
+
+	assert((nelems - rem) != 0);
+	{
+		const int nwords = ((nelems - rem) * X_SIZEOF_SHORT)
+					/ sizeof(word);
+		const word *const endw = &wp[nwords];
+
+#pragma _CRI ivdep
+		for( ; wp < endw; wp++)
 		{
-			cget_short_short(*xpp, tp, which);
-			*xpp = ((char *) (*xpp) + X_SIZEOF_SHORT);
-			nelems--;
-			if(nelems == 0)
-				return ENOERR;
+
+			*tp = (short)(*wp >> 48);
+			if(*tp & 0x8000)
+				*tp |= (~(0xffff));
+			tp++;
+
+			*tp = (short)((*wp >> 32) & 0xffff);
+			if(*tp & 0x8000)
+				*tp |= (~(0xffff));
+			tp++;
+
+			*tp = (short)((*wp >> 16) & 0xffff);
+			if(*tp & 0x8000)
+				*tp |= (~(0xffff));
+			tp++;
+
+			*tp = (short)(*wp & 0xffff);
+			if(*tp & 0x8000)
+				*tp |= (~(0xffff));
 			tp++;
 		}
 	}
 
-	assert(byteoff(*xpp) == 0);
-
+	if(tp <= last)
 	{
-		const int nwords = (nelems * X_SIZEOF_SHORT)/sizeof(word);
-		const word *wp = *xpp;
-		const word *const end = &wp[nwords];
-
-#pragma _CRI ivdep
-		for( ; wp < end; wp++, tp += 4)
-		{
-			cget_short_short(wp, tp, 0);
-			cget_short_short(wp, tp + 1, 1);
-			cget_short_short(wp, tp + 2, 2);
-			cget_short_short(wp, tp + 3, 3);
-		}
-
-		*xpp = ((char *) (*xpp) + nwords * sizeof(word)); 
-		nelems -= (nwords * sizeof(word)/X_SIZEOF_SHORT);
-		for(which = 0; nelems != 0; nelems--, which++, tp++) 
-		{
-			cget_short_short(*xpp, tp, which);
-			*xpp = ((char *) (*xpp) + X_SIZEOF_SHORT);
-		}
+		*tp = (short)(*wp >> 48);
+		if(*tp & 0x8000)
+			*tp |= (~(0xffff));
+		tp++;
+	}
+	if(tp <= last)
+	{
+		*tp = (short)((*wp >> 32) & 0xffff);
+		if(*tp & 0x8000)
+			*tp |= (~(0xffff));
+		tp++;
+	}
+	if(tp <= last)
+	{
+		*tp = (short)((*wp >> 16) & 0xffff);
+		if(*tp & 0x8000)
+			*tp |= (~(0xffff));
+		tp++;
 	}
 
+	}
 	return ENOERR;
 }
 #endif
@@ -2183,101 +2216,115 @@ ncx_putn_short_uchar(void **xpp, size_t nelems, const uchar *tp)
 int
 ncx_putn_short_short(void **xpp, size_t nelems, const short *tp)
 {
-	size_t noff = byteoff(*xpp);
-	int status = ENOERR;
-	int ierr;
-
-	if(noff != 0)
+	if(nelems > 0)
 	{
-		for(; nelems != 0 && noff % sizeof(word) != 0; 
-			nelems--, noff += X_SIZEOF_SHORT, tp++)
-		{
-			const int bitoff = noff * 8;
-			ierr = CRAY2IEG(&Cray2_I16, &nelems, (word *)(*xpp),
-				&bitoff, tp, &UnitStride);
-			assert(ierr >= 0);
+		word *wp = *xpp;
+		const int bo = bitoff(*xpp);
+		int ierr;
+	
+		*xpp = ((char *) (*xpp) + nelems * X_SIZEOF_SHORT);
+	
+		ierr = CRAY2IEG(&Cray2_I16, &nelems, wp,
+				&bo, tp, &UnitStride);
+		assert(ierr >= 0);
 		if(ierr > 0)
-			status = NC_ERANGE;
-		}
+			return NC_ERANGE;
 	}
-
-	if(nelems == 0)
-		return status;
-		
-	assert(byteoff(*xpp) == 0);
-
-	ierr = CRAY2IEG(&Cray2_I16, &nelems, (word *)(*xpp),
-			&Zero, tp, &UnitStride);
-	assert(ierr >= 0);
-
-	*xpp = (void *)((char *)(*xpp) + nelems * X_SIZEOF_SHORT);
-
-	if(ierr > 0)
-		status = NC_ERANGE;
-	return status;
+	return ENOERR;
 }
 #else
 int
-ncx_putn_short_short(void **xpp, size_t nelems, const short *tp)
+ncx_putn_short_short(void **xpp, const size_t nelems, const short *tp)
 {
 	int status = ENOERR;
-	int which = byteoff(*xpp)/X_SIZEOF_SHORT;
-
 	if(nelems == 0)
-		return ENOERR;
+		return status;
+{
+	word *wp = *xpp;
+	const short *const last = &tp[nelems -1];
+	const int rem = word_align(*xpp)/X_SIZEOF_SHORT;
+	*xpp = ((char *) (*xpp) + nelems * X_SIZEOF_SHORT);
 
-	if(which != 0)
+	switch(rem) {
+	case 3:
+		*wp = ((*tp << 32) & 0x0000ffff00000000)
+			    | (*wp & 0xffff0000ffffffff);
+		if(*tp > X_SHORT_MAX || *tp < X_SHORT_MIN)
+			status = NC_ERANGE;
+		if(tp == last)
+			return status;
+		tp++;
+		/*FALLTHRU*/	
+	case 2:
+		*wp = ((*tp << 16) & 0x00000000ffff0000)
+			    | (*wp & 0xffffffff0000ffff);
+		if(*tp > X_SHORT_MAX || *tp < X_SHORT_MIN)
+			status = NC_ERANGE;
+		if(tp == last)
+			return status;
+		tp++;
+		/*FALLTHRU*/	
+	case 1:
+		*wp = (*tp         & 0x000000000000ffff)   
+			    | (*wp & 0xffffffffffff0000);
+		if(*tp > X_SHORT_MAX || *tp < X_SHORT_MIN)
+			status = NC_ERANGE;
+		if(tp == last)
+			return status;
+		tp++;
+		wp++; /* Note Bene */
+		/*FALLTHRU*/	
+	}
+
+	assert((nelems - rem) != 0);
 	{
-		while(which++ < 4) 
+		const int nwords = ((nelems - rem) * X_SIZEOF_SHORT)
+					/ sizeof(word);
+		const word *const endw = &wp[nwords];
+
+#pragma _CRI ivdep
+		for( ; wp < endw; wp++)
 		{
-			const int lstatus = cput_short_short(*xpp, tp, which);
-			if(lstatus != ENOERR)
-				status = lstatus;
-			*xpp = ((char *) (*xpp) + X_SIZEOF_SHORT);
-			nelems--;
-			if(nelems == 0)
-				return ENOERR;
-			tp++;
+			*wp =	   (*tp      << 48)
+				| ((*(tp +1) << 32) & 0x0000ffff00000000)
+				| ((*(tp +2) << 16) & 0x00000000ffff0000)
+				| ((*(tp +3)      ) & 0x000000000000ffff);
+
+			{ int ii = 0;
+			for(; ii < 4; ii++, tp++)
+				if(*tp > X_SHORT_MAX || *tp < X_SHORT_MIN)
+					status = NC_ERANGE;
+			}
 		}
 	}
 
-	assert(byteoff(*xpp) == 0);
-
+	if(tp <= last)
 	{
-		const int nwords = (nelems * X_SIZEOF_SHORT)/sizeof(word);
-		const word *wp = *xpp;
-		const word *const end = &wp[nwords];
-
-#pragma _CRI ivdep
-		for( ; wp < end; wp++, tp += 4)
-		{
-			int lstatus = cput_short_short(wp, tp, 0);
-			if(lstatus != ENOERR)
-				status = lstatus;
-			lstatus = cput_short_short(wp, tp + 1, 1);
-			if(lstatus != ENOERR)
-				status = lstatus;
-			lstatus = cput_short_short(wp, tp + 2, 2);
-			if(lstatus != ENOERR)
-				status = lstatus;
-			lstatus = cput_short_short(wp, tp + 3, 3);
-			if(lstatus != ENOERR)
-				status = lstatus;
-		}
-
-		*xpp = ((char *) (*xpp) + nwords * sizeof(word)); 
-		nelems -= (nwords * sizeof(word)/X_SIZEOF_SHORT);
-		for(which = 0; nelems != 0; nelems--, which++, tp++) 
-		{
-			const int lstatus = cput_short_short(*xpp, tp, which);
-			if(lstatus != ENOERR)
-				status = lstatus;
-			*xpp = ((char *) (*xpp) + X_SIZEOF_SHORT);
-		}
+		*wp = (*tp << 48)
+			    | (*wp & 0x0000ffffffffffff);
+		if(*tp > X_SHORT_MAX || *tp < X_SHORT_MIN)
+			status = NC_ERANGE;
+		tp++;
+	}
+	if(tp <= last)
+	{
+		*wp = ((*tp << 32) & 0x0000ffff00000000)
+			    | (*wp & 0xffff0000ffffffff);
+		if(*tp > X_SHORT_MAX || *tp < X_SHORT_MIN)
+			status = NC_ERANGE;
+		tp++;
+	}
+	if(tp <= last)
+	{
+		*wp = ((*tp << 16) & 0x00000000ffff0000)
+			    | (*wp & 0xffffffff0000ffff);
+		if(*tp > X_SHORT_MAX || *tp < X_SHORT_MIN)
+			status = NC_ERANGE;
 	}
 
 	return status;
 }
+}
 #endif
 
 int
@@ -2573,39 +2620,19 @@ ncx_getn_int_short(const void **xpp, size_t nelems, short *tp)
 int
 ncx_getn_int_int(const void **xpp, size_t nelems, int *tp)
 {
-	const size_t noff = word_align(*xpp);
-	int status = ENOERR;
-	int ierr;
-
-	if(nelems != 0 && noff != 0)
+	if(nelems > 0)
 	{
-		assert(noff == X_SIZEOF_INT);
-		ierr = IEG2CRAY(&Cray2_I32, &One, (word *)(*xpp),
-			&ThirtyTwo, tp, &UnitStride);
+		const int bo = bitoff(*xpp);
+		const word *wp = *xpp;
+		int ierr;
+		*xpp = ((const char *) (*xpp) + nelems * X_SIZEOF_INT);
+		ierr = IEG2CRAY(&Cray2_I32, &nelems, wp,
+				&bo, tp, &UnitStride);
 		assert(ierr >= 0);
-
-		*xpp = (void *)((char *)(*xpp) + X_SIZEOF_INT);
-		tp++;
-		nelems--;
-
 		if(ierr > 0)
-			status = NC_ERANGE;
+			return NC_ERANGE;
 	}
-
-	if(nelems == 0)
-		return status;
-		
-	assert(word_align(*xpp) == 0);
-
-	ierr = IEG2CRAY(&Cray2_I32, &nelems, (word *)(*xpp),
-			&Zero, tp, &UnitStride);
-	assert(ierr >= 0);
-
-	*xpp = (void *)((char *)(*xpp) + nelems * X_SIZEOF_INT);
-
-	if(ierr > 0)
-		status = NC_ERANGE;
-	return status;
+	return ENOERR;
 }
 #else
 int
@@ -2787,39 +2814,21 @@ ncx_putn_int_short(void **xpp, size_t nelems, const short *tp)
 int
 ncx_putn_int_int(void **xpp, size_t nelems, const int *tp)
 {
-	const size_t noff = word_align(*xpp);
-	int status = ENOERR;
-	int ierr;
-
-	if(nelems != 0 && noff != 0)
+	if(nelems > 0)
 	{
-		assert(noff == X_SIZEOF_INT);
-		ierr = CRAY2IEG(&Cray2_I32, &One, (word *)(*xpp),
-			&ThirtyTwo, tp, &UnitStride);
+		word *wp = *xpp;
+		const int bo = bitoff(*xpp);
+		int ierr;
+	
+		*xpp = ((char *) (*xpp) + nelems * X_SIZEOF_INT);
+	
+		ierr = CRAY2IEG(&Cray2_I32, &nelems, wp,
+				&bo, tp, &UnitStride);
 		assert(ierr >= 0);
-
-		*xpp = (void *)((char *)(*xpp) + X_SIZEOF_INT);
-		tp++;
-		nelems--;
-
 		if(ierr > 0)
-			status = NC_ERANGE;
+			return NC_ERANGE;
 	}
-
-	if(nelems == 0)
-		return status;
-		
-	assert(word_align(*xpp) == 0);
-
-	ierr = CRAY2IEG(&Cray2_I32, &nelems, (word *)(*xpp),
-			&Zero, tp, &UnitStride);
-	assert(ierr >= 0);
-
-	*xpp = (void *)((char *)(*xpp) + nelems * X_SIZEOF_INT);
-
-	if(ierr > 0)
-		status = NC_ERANGE;
-	return status;
+	return ENOERR;
 }
 #else
 int
@@ -3052,39 +3061,20 @@ ncx_getn_float_long(const void **xpp, size_t nelems, long *tp)
 int
 ncx_getn_float_float(const void **xpp, size_t nelems, float *tp)
 {
-	const size_t noff = word_align(*xpp);
-	int status = ENOERR;
-	int ierr;
-
-	if(nelems != 0 && noff != 0)
+	if(nelems > 0)
 	{
-		assert(noff == X_SIZEOF_FLOAT);
-		ierr = IEG2CRAY(&Cray2_F32, &One, (word *)(*xpp),
-			&ThirtyTwo, tp, &UnitStride);
+		const int bo = bitoff(*xpp);
+		const word *wp = *xpp;
+		int ierr;
+		*xpp = ((const char *) (*xpp) + nelems * X_SIZEOF_FLOAT);
+		ierr = IEG2CRAY(&Cray2_F32, &nelems, wp,
+				&bo, tp, &UnitStride);
 		assert(ierr >= 0);
-
-		*xpp = (void *)((char *)(*xpp) + X_SIZEOF_FLOAT);
-		tp++;
-		nelems--;
-
 		if(ierr > 0)
-			status = NC_ERANGE;
+			return NC_ERANGE;
 	}
+	return ENOERR;
 
-	if(nelems == 0)
-		return status;
-		
-	assert(word_align(*xpp) == 0);
-
-	ierr = IEG2CRAY(&Cray2_F32, &nelems, (word *)(*xpp),
-			&Zero, tp, &UnitStride);
-	assert(ierr >= 0);
-
-	*xpp = (void *)((char *)(*xpp) + nelems * X_SIZEOF_FLOAT);
-
-	if(ierr > 0)
-		status = NC_ERANGE;
-	return status;
 }
 #else
 int
@@ -3239,39 +3229,21 @@ ncx_putn_float_long(void **xpp, size_t nelems, const long *tp)
 int
 ncx_putn_float_float(void **xpp, size_t nelems, const float *tp)
 {
-	const size_t noff = word_align(*xpp);
-	int status = ENOERR;
-	int ierr;
-
-	if(nelems != 0 && noff != 0)
+	if(nelems > 0)
 	{
-		assert(noff == X_SIZEOF_FLOAT);
-		ierr = CRAY2IEG(&Cray2_F32, &One, (word *)(*xpp),
-			&ThirtyTwo, tp, &UnitStride);
+		word *wp = *xpp;
+		const int bo = bitoff(*xpp);
+		int ierr;
+	
+		*xpp = ((char *) (*xpp) + nelems * X_SIZEOF_FLOAT);
+	
+		ierr = CRAY2IEG(&Cray2_F32, &nelems, wp,
+				&bo, tp, &UnitStride);
 		assert(ierr >= 0);
-
-		*xpp = (void *)((char *)(*xpp) + X_SIZEOF_FLOAT);
-		tp++;
-		nelems--;
-
 		if(ierr > 0)
-			status = NC_ERANGE;
+			return NC_ERANGE;
 	}
-
-	if(nelems == 0)
-		return status;
-		
-	assert(byteoff(*xpp) == 0);
-
-	ierr = CRAY2IEG(&Cray2_F32, &nelems, (word *)(*xpp),
-			&Zero, tp, &UnitStride);
-	assert(ierr >= 0);
-
-	*xpp = (void *)((char *)(*xpp) + nelems * X_SIZEOF_FLOAT);
-
-	if(ierr > 0)
-		status = NC_ERANGE;
-	return status;
+	return ENOERR;
 }
 #else
 int
@@ -3439,6 +3411,9 @@ ncx_getn_double_double(const void **xpp, size_t nelems, double *tp)
 	const size_t noff = byteoff(*xpp);
 	int ierr;
 
+	if(nelems == 0)
+		return ENOERR;
+
 	if(noff != 0)
 	{
 		/* (*xpp) not word aligned, forced to make a copy */
@@ -3453,10 +3428,10 @@ ncx_getn_double_double(const void **xpp, size_t nelems, double *tp)
 				&Zero, tp, &UnitStride);
 	}
 
-	assert(ierr >= 0);
-
 	*xpp = (void *)((char *)(*xpp) + nelems * X_SIZEOF_DOUBLE);
 
+	assert(ierr >= 0);
+
 	return ierr > 0 ? NC_ERANGE : ENOERR;
 }
 #elif X_SIZEOF_DOUBLE == SIZEOF_DOUBLE  && !defined(NO_IEEE_FLOAT)
@@ -3623,6 +3598,9 @@ ncx_putn_double_double(void **xpp, size_t nelems, const double *tp)
 	const size_t noff = byteoff(*xpp);
 	int ierr;
 
+	if(nelems == 0)
+		return ENOERR;
+
 	if(noff != 0)
 	{
 		/* (*xpp) not word aligned, forced to make a copy */
