@@ -30,7 +30,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/* "$Id: netcdf.h,v 2.69 1997/06/10 04:05:27 davis Exp $" */
+/* "$Id: netcdf.h,v 2.70 1997/10/06 20:12:15 davis Exp $" */
 
 #ifndef _NETCDF_
 #define _NETCDF_
@@ -91,6 +91,17 @@ typedef enum {
 #define NC_LOCK		0x0400	/* Use locking if available */
 #define NC_SHARE	0x0800	/* Share updates, limit cacheing */
 
+/*
+ * Let nc__create() or nc__open() figure out
+ * as suitable chunk size.
+ */
+#define NC_SIZEHINT_DEFAULT 0
+
+/*
+ * In nc__enddef(), align to the chunk size.
+ */
+#define NC_ALIGN_CHUNK ((size_t)(-1))
+
 /*
  * 'size' argument to ncdimdef for an unlimited dimension
  */
@@ -180,9 +191,17 @@ nc_inq_libvers(void);
 EXTERNL const char *
 nc_strerror(int ncerr);
 
+EXTERNL int
+nc__create(const char *path, int cmode, size_t initialsz,
+	 size_t *chunksizehintp, int *ncidp);
+
 EXTERNL int
 nc_create(const char *path, int cmode, int *ncidp);
 
+EXTERNL int
+nc__open(const char *path, int mode, 
+	size_t *chunksizehintp, int *ncidp);
+
 EXTERNL int
 nc_open(const char *path, int mode, int *ncidp);
 
@@ -192,6 +211,10 @@ nc_set_fill(int ncid, int fillmode, int *old_modep);
 EXTERNL int
 nc_redef(int ncid);
 
+EXTERNL int
+nc__enddef(int ncid, size_t h_minfree, size_t v_align,
+	size_t v_minfree, size_t r_align);
+
 EXTERNL int
 nc_enddef(int ncid);
 
