@@ -2,7 +2,7 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: v1hpg.c,v 1.44 1997/06/04 20:07:53 davis Exp $ */
+/* $Id: v1hpg.c,v 1.49 1997/12/18 20:03:28 davis Exp $ */
 
 #include "nc.h"
 #include <stdlib.h>
@@ -31,34 +31,36 @@ static const schar ncmagic[] = {'C', 'D', 'F', 0x01};
 
 
 /*
- * v1hgs == "Version 1 Header Get Stream"
+ * v1hs == "Version 1 Header Stream"
  *
  * The netcdf file version 1 header is
  * of unknown and potentially unlimited size.
  * So, we don't know how much to get() on
- * the initial read. We build a stream, 'v1hgs'
+ * the initial read. We build a stream, 'v1hs'
  * on top of ncio to do the header get.
  */
-typedef struct v1hgs {
+typedef struct v1hs {
 	ncio *nciop;
 	off_t offset;	/* argument to nciop->get() */
 	size_t extent;	/* argument to nciop->get() */
+	int flags;	/* set to RGN_WRITE for write */
 	void *base;	/* beginning of current buffer */
-	const void *pos;	/* current position in buffer */
+	void *pos;	/* current position in buffer */
 	void *end;	/* end of current buffer = base + extent */
-} v1hgs;
+} v1hs;
 
 
 /*
  * Release the stream, invalidate buffer
  */
 static int
-rel_v1hgs(v1hgs *gsp)
+rel_v1hs(v1hs *gsp)
 {
 	int status;
 	if(gsp->offset == OFF_NONE || gsp->base == NULL)
 		return ENOERR;
-	status = gsp->nciop->rel(gsp->nciop, gsp->offset, 0);
+	status = gsp->nciop->rel(gsp->nciop, gsp->offset,
+			 gsp->flags == RGN_WRITE ? RGN_MODIFIED : 0);
 	gsp->end = NULL;
 	gsp->pos = NULL;
 	gsp->base = NULL;
@@ -71,14 +73,14 @@ rel_v1hgs(v1hgs *gsp)
  * Also used for initialization when gsp->base == NULL.
  */
 static int
-fault_v1hgs(v1hgs *gsp, size_t extent)
+fault_v1hs(v1hs *gsp, size_t extent)
 {
 	int status;
 
 	if(gsp->base != NULL)
 	{
 		const size_t incr = (char *)gsp->pos - (char *)gsp->base;
-		status = rel_v1hgs(gsp);
+		status = rel_v1hs(gsp);
 		if(status)
 			return status;
 		gsp->offset += incr;
@@ -89,7 +91,7 @@ fault_v1hgs(v1hgs *gsp, size_t extent)
 
 	status = gsp->nciop->get(gsp->nciop,
 		 	gsp->offset, gsp->extent,
-			0, &gsp->base);
+			gsp->flags, &gsp->base);
 	if(status)
 		return status;
 
@@ -104,7 +106,7 @@ fault_v1hgs(v1hgs *gsp, size_t extent)
  * Ensure that 'nextread' bytes are available.
  */
 static int
-check_v1hgs(v1hgs *gsp, size_t nextread)
+check_v1hs(v1hs *gsp, size_t nextread)
 {
 
 #if 0 /* DEBUG */
@@ -115,19 +117,28 @@ fprintf(stderr, "nextread %lu, remaining %lu\n",
 
 	if((char *)gsp->pos + nextread < (char *)gsp->end)
 		return ENOERR;
-	return fault_v1hgs(gsp, nextread);
+	return fault_v1hs(gsp, nextread);
 }
 
-/* End v1hgs */
+/* End v1hs */
+
+static int
+v1h_put_size_t(v1hs *psp, const size_t *sp)
+{
+	int status = check_v1hs(psp, X_SIZEOF_SIZE_T);
+	if(status != ENOERR)
+		return status;
+	return ncx_put_size_t(&psp->pos, sp);
+}
 
 
 static int
-v1h_get_size_t(v1hgs *gsp, size_t *sp)
+v1h_get_size_t(v1hs *gsp, size_t *sp)
 {
-	int status = check_v1hgs(gsp, X_SIZEOF_SIZE_T);
+	int status = check_v1hs(gsp, X_SIZEOF_SIZE_T);
 	if(status != ENOERR)
 		return status;
-	return ncx_get_size_t(&gsp->pos, sp);
+	return ncx_get_size_t((const void **)(&gsp->pos), sp);
 }
 
 
@@ -136,20 +147,23 @@ v1h_get_size_t(v1hgs *gsp, size_t *sp)
 #define X_SIZEOF_NC_TYPE X_SIZEOF_INT
 
 static int
-ncx_put_nc_type(void **xpp, const nc_type *typep)
+v1h_put_nc_type(v1hs *psp, const nc_type *typep)
 {
 	const int itype = (int) *typep;
-	const int status =  ncx_put_int_int(*xpp, &itype);
-	*xpp = (void *)((char *)(*xpp) + X_SIZEOF_INT);
+	int status = check_v1hs(psp, X_SIZEOF_INT);
+	if(status != ENOERR)
+		return status;
+	status =  ncx_put_int_int(psp->pos, &itype);
+	psp->pos = (void *)((char *)psp->pos + X_SIZEOF_INT);
 	return status;
 }
 
 
 static int
-v1h_get_nc_type(v1hgs *gsp, nc_type *typep)
+v1h_get_nc_type(v1hs *gsp, nc_type *typep)
 {
 	int type = 0;
-	int status = check_v1hgs(gsp, X_SIZEOF_INT);
+	int status = check_v1hs(gsp, X_SIZEOF_INT);
 	if(status != ENOERR)
 		return status;
 	status =  ncx_get_int_int(gsp->pos, &type);
@@ -176,19 +190,22 @@ v1h_get_nc_type(v1hgs *gsp, nc_type *typep)
 #define X_SIZEOF_NCTYPE X_SIZEOF_INT
 
 static int
-ncx_put_NCtype(void **xpp, NCtype type)
+v1h_put_NCtype(v1hs *psp, NCtype type)
 {
 	const int itype = (int) type;
-	const int status = ncx_put_int_int(*xpp, &itype);
-	*xpp = (void *)((char *)(*xpp) + X_SIZEOF_INT);
+	int status = check_v1hs(psp, X_SIZEOF_INT);
+	if(status != ENOERR)
+		return status;
+	status = ncx_put_int_int(psp->pos, &itype);
+	psp->pos = (void *)((char *)psp->pos + X_SIZEOF_INT);
 	return status;
 }
 
 static int
-v1h_get_NCtype(v1hgs *gsp, NCtype *typep)
+v1h_get_NCtype(v1hs *gsp, NCtype *typep)
 {
 	int type = 0;
-	int status = check_v1hgs(gsp, X_SIZEOF_INT);
+	int status = check_v1hs(gsp, X_SIZEOF_INT);
 	if(status != ENOERR)
 		return status;
 	status =  ncx_get_int_int(gsp->pos, &type);
@@ -229,7 +246,7 @@ ncx_len_NC_string(const NC_string *ncstrp)
 
 
 static int
-ncx_put_NC_string(void **xpp, const NC_string *ncstrp)
+v1h_put_NC_string(v1hs *psp, const NC_string *ncstrp)
 {
 	int status;
 
@@ -237,10 +254,13 @@ ncx_put_NC_string(void **xpp, const NC_string *ncstrp)
 	assert(ncstrp->nchars % X_ALIGN == 0);
 #endif
 
-	status = ncx_put_size_t(xpp, &ncstrp->nchars);
+	status = v1h_put_size_t(psp, &ncstrp->nchars);
 	if(status != ENOERR)
 		return status;
-	status = ncx_pad_putn_text(xpp, ncstrp->nchars, ncstrp->cp);
+	status = check_v1hs(psp, _RNDUP(ncstrp->nchars, X_ALIGN));
+	if(status != ENOERR)
+		return status;
+	status = ncx_pad_putn_text(&psp->pos, ncstrp->nchars, ncstrp->cp);
 	if(status != ENOERR)
 		return status;
 
@@ -249,7 +269,7 @@ ncx_put_NC_string(void **xpp, const NC_string *ncstrp)
 
 
 static int
-v1h_get_NC_string(v1hgs *gsp, NC_string **ncstrpp)
+v1h_get_NC_string(v1hs *gsp, NC_string **ncstrpp)
 {
 	int status;
 	size_t nchars = 0;
@@ -269,15 +289,16 @@ v1h_get_NC_string(v1hgs *gsp, NC_string **ncstrpp)
 #if 0
 /* assert(ncstrp->nchars == nchars || ncstrp->nchars - nchars < X_ALIGN); */
 	assert(ncstrp->nchars % X_ALIGN == 0);
-	status = check_v1hgs(gsp, ncstrp->nchars);
+	status = check_v1hs(gsp, ncstrp->nchars);
 #else
 	
-	status = check_v1hgs(gsp, _RNDUP(ncstrp->nchars, X_ALIGN));
+	status = check_v1hs(gsp, _RNDUP(ncstrp->nchars, X_ALIGN));
 #endif
 	if(status != ENOERR)
 		goto unwind_alloc;
 
-	status = ncx_pad_getn_text(&gsp->pos, nchars, ncstrp->cp);
+	status = ncx_pad_getn_text((const void **)(&gsp->pos),
+		 nchars, ncstrp->cp);
 	if(status != ENOERR)
 		goto unwind_alloc;
 
@@ -314,15 +335,15 @@ ncx_len_NC_dim(const NC_dim *dimp)
 
 
 static int
-ncx_put_NC_dim(void **xpp, const NC_dim *dimp)
+v1h_put_NC_dim(v1hs *psp, const NC_dim *dimp)
 {
 	int status;
 
-	status = ncx_put_NC_string(xpp, dimp->name);
+	status = v1h_put_NC_string(psp, dimp->name);
 	if(status != ENOERR)
 		return status;
 
-	status = ncx_put_size_t(xpp, &dimp->size);
+	status = v1h_put_size_t(psp, &dimp->size);
 	if(status != ENOERR)
 		return status;
 
@@ -330,7 +351,7 @@ ncx_put_NC_dim(void **xpp, const NC_dim *dimp)
 }
 
 static int
-v1h_get_NC_dim(v1hgs *gsp, NC_dim **dimpp)
+v1h_get_NC_dim(v1hs *gsp, NC_dim **dimpp)
 {
 	int status;
 	NC_string *ncstrp;
@@ -385,11 +406,11 @@ ncx_len_NC_dimarray(const NC_dimarray *ncap)
 
 
 static int
-ncx_put_NC_dimarray(void **xpp, const NC_dimarray *ncap)
+v1h_put_NC_dimarray(v1hs *psp, const NC_dimarray *ncap)
 {
 	int status;
 
-	assert(*xpp != NULL);
+	assert(psp != NULL);
 
 	if(ncap == NULL
 #if 1
@@ -407,20 +428,20 @@ ncx_put_NC_dimarray(void **xpp, const NC_dimarray *ncap)
 		 */
 		const size_t nosz = 0;
 
-		status = ncx_put_NCtype(xpp, NC_UNSPECIFIED);
+		status = v1h_put_NCtype(psp, NC_UNSPECIFIED);
 		if(status != ENOERR)
 			return status;
-		status = ncx_put_size_t(xpp, &nosz);
+		status = v1h_put_size_t(psp, &nosz);
 		if(status != ENOERR)
 			return status;
 		return ENOERR;
 	}
 	/* else */
 
-	status = ncx_put_NCtype(xpp, NC_DIMENSION);
+	status = v1h_put_NCtype(psp, NC_DIMENSION);
 	if(status != ENOERR)
 		return status;
-	status = ncx_put_size_t(xpp, &ncap->nelems);
+	status = v1h_put_size_t(psp, &ncap->nelems);
 	if(status != ENOERR)
 		return status;
 
@@ -429,7 +450,7 @@ ncx_put_NC_dimarray(void **xpp, const NC_dimarray *ncap)
 		const NC_dim *const *const end = &dpp[ncap->nelems];
 		for( /*NADA*/; dpp < end; dpp++)
 		{
-			status = ncx_put_NC_dim(xpp, *dpp);
+			status = v1h_put_NC_dim(psp, *dpp);
 			if(status)
 				return status;
 		}
@@ -439,7 +460,7 @@ ncx_put_NC_dimarray(void **xpp, const NC_dimarray *ncap)
 
 
 static int
-v1h_get_NC_dimarray(v1hgs *gsp, NC_dimarray *ncap)
+v1h_get_NC_dimarray(v1hs *gsp, NC_dimarray *ncap)
 {
 	int status;
 	NCtype type = NC_UNSPECIFIED;
@@ -511,32 +532,67 @@ ncx_len_NC_attr(const NC_attr *attrp)
 }
 
 
+#undef MIN
+#define MIN(mm,nn) (((mm) < (nn)) ? (mm) : (nn))
+
+/*
+ * Put the values of an attribute
+ * The loop is necessary since attrp->nelems
+ * could potentially be quite large.
+ */
 static int
-ncx_put_NC_attr(void **xpp, const NC_attr *attrp)
+v1h_put_NC_attrV(v1hs *psp, const NC_attr *attrp)
 {
 	int status;
+	const size_t perchunk =  psp->extent;
+	size_t remaining = attrp->xsz;
+	void *value = attrp->xvalue;
+	size_t nbytes; 
 
-	status = ncx_put_NC_string(xpp, attrp->name);
-	if(status != ENOERR)
-		return status;
+	assert(psp->extent % X_ALIGN == 0);
+	
+	do {
+		nbytes = MIN(perchunk, remaining);
+	
+		status = check_v1hs(psp, nbytes);
+		if(status != ENOERR)
+			return status;
+	
+		(void) memcpy(psp->pos, value, nbytes);
 
-	status = ncx_put_nc_type(xpp, &attrp->type);
-	if(status != ENOERR)
-		return status;
+		psp->pos = (void *)((char *)psp->pos + nbytes);
+		value = (void *)((char *)value + nbytes);
+		remaining -= nbytes;
 
-	status = ncx_put_size_t(xpp, &attrp->nelems);
-	if(status != ENOERR)
-		return status;
-
-	(void) memcpy(*xpp, attrp->xvalue, attrp->xsz);
-	*xpp = (void *)((char *)(*xpp) + attrp->xsz);
+	} while(remaining != 0); 
 
 	return ENOERR;
 }
 
+static int
+v1h_put_NC_attr(v1hs *psp, const NC_attr *attrp)
+{
+	int status;
+
+	status = v1h_put_NC_string(psp, attrp->name);
+	if(status != ENOERR)
+		return status;
+
+	status = v1h_put_nc_type(psp, &attrp->type);
+	if(status != ENOERR)
+		return status;
+
+	status = v1h_put_size_t(psp, &attrp->nelems);
+	if(status != ENOERR)
+		return status;
+
+	status = v1h_put_NC_attrV(psp, attrp);
+	if(status != ENOERR)
+		return status;
+
+	return ENOERR;
+}
 
-#undef MIN
-#define MIN(mm,nn) (((mm) < (nn)) ? (mm) : (nn))
 
 /*
  * Get the values of an attribute
@@ -544,7 +600,7 @@ ncx_put_NC_attr(void **xpp, const NC_attr *attrp)
  * could potentially be quite large.
  */
 static int
-v1h_get_NC_attrV(v1hgs *gsp, NC_attr *attrp)
+v1h_get_NC_attrV(v1hs *gsp, NC_attr *attrp)
 {
 	int status;
 	const size_t perchunk =  gsp->extent;
@@ -557,7 +613,7 @@ v1h_get_NC_attrV(v1hgs *gsp, NC_attr *attrp)
 	do {
 		nget = MIN(perchunk, remaining);
 	
-		status = check_v1hgs(gsp, nget);
+		status = check_v1hs(gsp, nget);
 		if(status != ENOERR)
 			return status;
 	
@@ -574,7 +630,7 @@ v1h_get_NC_attrV(v1hgs *gsp, NC_attr *attrp)
 
 
 static int
-v1h_get_NC_attr(v1hgs *gsp, NC_attr **attrpp)
+v1h_get_NC_attr(v1hs *gsp, NC_attr **attrpp)
 {
 	NC_string *strp;
 	int status;
@@ -639,11 +695,11 @@ ncx_len_NC_attrarray(const NC_attrarray *ncap)
 
 
 static int
-ncx_put_NC_attrarray(void **xpp, const NC_attrarray *ncap)
+v1h_put_NC_attrarray(v1hs *psp, const NC_attrarray *ncap)
 {
 	int status;
 
-	assert(*xpp != NULL);
+	assert(psp != NULL);
 
 	if(ncap == NULL
 #if 1
@@ -661,20 +717,20 @@ ncx_put_NC_attrarray(void **xpp, const NC_attrarray *ncap)
 		 */
 		const size_t nosz = 0;
 
-		status = ncx_put_NCtype(xpp, NC_UNSPECIFIED);
+		status = v1h_put_NCtype(psp, NC_UNSPECIFIED);
 		if(status != ENOERR)
 			return status;
-		status = ncx_put_size_t(xpp, &nosz);
+		status = v1h_put_size_t(psp, &nosz);
 		if(status != ENOERR)
 			return status;
 		return ENOERR;
 	}
 	/* else */
 
-	status = ncx_put_NCtype(xpp, NC_ATTRIBUTE);
+	status = v1h_put_NCtype(psp, NC_ATTRIBUTE);
 	if(status != ENOERR)
 		return status;
-	status = ncx_put_size_t(xpp, &ncap->nelems);
+	status = v1h_put_size_t(psp, &ncap->nelems);
 	if(status != ENOERR)
 		return status;
 
@@ -683,7 +739,7 @@ ncx_put_NC_attrarray(void **xpp, const NC_attrarray *ncap)
 		const NC_attr *const *const end = &app[ncap->nelems];
 		for( /*NADA*/; app < end; app++)
 		{
-			status = ncx_put_NC_attr(xpp, *app);
+			status = v1h_put_NC_attr(psp, *app);
 			if(status)
 				return status;
 		}
@@ -693,7 +749,7 @@ ncx_put_NC_attrarray(void **xpp, const NC_attrarray *ncap)
 
 
 static int
-v1h_get_NC_attrarray(v1hgs *gsp, NC_attrarray *ncap)
+v1h_get_NC_attrarray(v1hs *gsp, NC_attrarray *ncap)
 {
 	int status;
 	NCtype type = NC_UNSPECIFIED;
@@ -766,35 +822,42 @@ ncx_len_NC_var(const NC_var *varp)
 
 
 static int
-ncx_put_NC_var(void **xpp, const NC_var *varp)
+v1h_put_NC_var(v1hs *psp, const NC_var *varp)
 {
 	int status;
 
-	status = ncx_put_NC_string(xpp, varp->name);
+	status = v1h_put_NC_string(psp, varp->name);
 	if(status != ENOERR)
 		return status;
 
-	status = ncx_put_size_t(xpp, &varp->ndims);
+	status = v1h_put_size_t(psp, &varp->ndims);
 	if(status != ENOERR)
 		return status;
 
-	status = ncx_putn_int_int(xpp, varp->ndims, varp->dimids);
+	status = check_v1hs(psp, ncx_len_int(varp->ndims));
+	if(status != ENOERR)
+		return status;
+	status = ncx_putn_int_int(&psp->pos,
+			varp->ndims, varp->dimids);
 	if(status != ENOERR)
 		return status;
 
-	status = ncx_put_NC_attrarray(xpp, &varp->attrs);
+	status = v1h_put_NC_attrarray(psp, &varp->attrs);
 	if(status != ENOERR)
 		return status;
 
-	status = ncx_put_nc_type(xpp, &varp->type);
+	status = v1h_put_nc_type(psp, &varp->type);
 	if(status != ENOERR)
 		return status;
 
-	status = ncx_put_size_t(xpp, &varp->len);
+	status = v1h_put_size_t(psp, &varp->len);
 	if(status != ENOERR)
 		return status;
 
-	status = ncx_put_off_t(xpp, &varp->begin);
+	status = check_v1hs(psp, X_SIZEOF_OFF_T);
+	if(status != ENOERR)
+		 return status;
+	status = ncx_put_off_t(&psp->pos, &varp->begin);
 	if(status != ENOERR)
 		return status;
 
@@ -803,7 +866,7 @@ ncx_put_NC_var(void **xpp, const NC_var *varp)
 
 
 static int
-v1h_get_NC_var(v1hgs *gsp, NC_var **varpp)
+v1h_get_NC_var(v1hs *gsp, NC_var **varpp)
 {
 	NC_string *strp;
 	int status;
@@ -825,10 +888,11 @@ v1h_get_NC_var(v1hgs *gsp, NC_var **varpp)
 		goto unwind_name;
 	}
 
-	status = check_v1hgs(gsp, ncx_len_int(ndims));
+	status = check_v1hs(gsp, ncx_len_int(ndims));
 	if(status != ENOERR)
 		goto unwind_alloc;
-	status = ncx_getn_int_int(&gsp->pos, ndims, varp->dimids);
+	status = ncx_getn_int_int((const void **)(&gsp->pos),
+			ndims, varp->dimids);
 	if(status != ENOERR)
 		goto unwind_alloc;
 
@@ -844,10 +908,11 @@ v1h_get_NC_var(v1hgs *gsp, NC_var **varpp)
 	if(status != ENOERR)
 		 goto unwind_alloc;
 
-	status = check_v1hgs(gsp, X_SIZEOF_OFF_T);
+	status = check_v1hs(gsp, X_SIZEOF_OFF_T);
 	if(status != ENOERR)
 		 goto unwind_alloc;
-	status = ncx_get_off_t(&gsp->pos, &varp->begin);
+	status = ncx_get_off_t((const void **)&gsp->pos,
+			&varp->begin);
 	if(status != ENOERR)
 		 goto unwind_alloc;
 	
@@ -885,11 +950,11 @@ ncx_len_NC_vararray(const NC_vararray *ncap)
 
 
 static int
-ncx_put_NC_vararray(void **xpp, const NC_vararray *ncap)
+v1h_put_NC_vararray(v1hs *psp, const NC_vararray *ncap)
 {
 	int status;
 
-	assert(*xpp != NULL);
+	assert(psp != NULL);
 
 	if(ncap == NULL
 #if 1
@@ -907,20 +972,20 @@ ncx_put_NC_vararray(void **xpp, const NC_vararray *ncap)
 		 */
 		const size_t nosz = 0;
 
-		status = ncx_put_NCtype(xpp, NC_UNSPECIFIED);
+		status = v1h_put_NCtype(psp, NC_UNSPECIFIED);
 		if(status != ENOERR)
 			return status;
-		status = ncx_put_size_t(xpp, &nosz);
+		status = v1h_put_size_t(psp, &nosz);
 		if(status != ENOERR)
 			return status;
 		return ENOERR;
 	}
 	/* else */
 
-	status = ncx_put_NCtype(xpp, NC_VARIABLE);
+	status = v1h_put_NCtype(psp, NC_VARIABLE);
 	if(status != ENOERR)
 		return status;
-	status = ncx_put_size_t(xpp, &ncap->nelems);
+	status = v1h_put_size_t(psp, &ncap->nelems);
 	if(status != ENOERR)
 		return status;
 
@@ -929,7 +994,7 @@ ncx_put_NC_vararray(void **xpp, const NC_vararray *ncap)
 		const NC_var *const *const end = &vpp[ncap->nelems];
 		for( /*NADA*/; vpp < end; vpp++)
 		{
-			status = ncx_put_NC_var(xpp, *vpp);
+			status = v1h_put_NC_var(psp, *vpp);
 			if(status)
 				return status;
 		}
@@ -939,7 +1004,7 @@ ncx_put_NC_vararray(void **xpp, const NC_vararray *ncap)
 
 
 static int
-v1h_get_NC_vararray(v1hgs *gsp, NC_vararray *ncap)
+v1h_get_NC_vararray(v1hs *gsp, NC_vararray *ncap)
 {
 	int status;
 	NCtype type = NC_UNSPECIFIED;
@@ -1083,34 +1148,82 @@ ncx_len_NC(const NC *ncp)
 
 
 int
-ncx_put_NC(void **xpp, const NC *ncp)
+ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent)
 {
-	int status;
+	int status = ENOERR;
+	v1hs ps; /* the get stream */
 
-	assert(*xpp != NULL);
 	assert(ncp != NULL);
 
-	status = ncx_putn_schar_schar(xpp, sizeof(ncmagic), ncmagic);
-	if(status != ENOERR)
-		return status;
+	/* Initialize stream ps */
 
-	status = ncx_put_size_t(xpp, &ncp->numrecs);
-	if(status != ENOERR)
-		return status;
+	ps.nciop = ncp->nciop;
+	ps.flags = RGN_WRITE;
 
-	status = ncx_put_NC_dimarray(xpp, &ncp->dims);
-	if(status != ENOERR)
-		return status;
+	if(xpp == NULL)
+	{
+		/*
+		 * Come up with a reasonable stream read size.
+		 */
+		extent = ncp->xsz;
+		if(extent <= MIN_NC_XSZ)
+		{
+			/* first time read */
+			extent = ncp->chunk;
+			/* Protection for when ncp->chunk is huge;
+			 * no need to read hugely. */
+	      		if(extent > 4096)
+				extent = 4096;
+		}
+		else if(extent > ncp->chunk)
+		{
+			extent = ncp->chunk;
+		}
+		
+		ps.offset = 0;
+		ps.extent = extent;
+		ps.base = NULL;
+		ps.pos = ps.base;
 
-	status = ncx_put_NC_attrarray(xpp, &ncp->attrs);
-	if(status != ENOERR)
-		return status;
+		status = fault_v1hs(&ps, extent);
+		if(status)
+			return status;
+	}
+	else
+	{
+		ps.offset = offset;
+		ps.extent = extent;
+		ps.base = *xpp;
+		ps.pos = ps.base;
+		ps.end = (char *)ps.base + ps.extent;
+	}
 
-	status = ncx_put_NC_vararray(xpp, &ncp->vars);
+	status = ncx_putn_schar_schar(&ps.pos, sizeof(ncmagic), ncmagic);
 	if(status != ENOERR)
-		return status;
+		goto release;
 
-	return ENOERR;
+	status = ncx_put_size_t(&ps.pos, &ncp->numrecs);
+	if(status != ENOERR)
+		goto release;
+
+	assert((char *)ps.pos < (char *)ps.end);
+
+	status = v1h_put_NC_dimarray(&ps, &ncp->dims);
+	if(status != ENOERR)
+		goto release;
+
+	status = v1h_put_NC_attrarray(&ps, &ncp->attrs);
+	if(status != ENOERR)
+		goto release;
+
+	status = v1h_put_NC_vararray(&ps, &ncp->vars);
+	if(status != ENOERR)
+		goto release;
+
+release:
+	(void) rel_v1hs(&ps);
+
+	return status;
 }
 
 
@@ -1118,7 +1231,7 @@ int
 nc_get_NC(NC *ncp)
 {
 	int status;
-	v1hgs gs;
+	v1hs gs; /* the get stream */
 
 	assert(ncp != NULL);
 
@@ -1127,7 +1240,9 @@ nc_get_NC(NC *ncp)
 	gs.nciop = ncp->nciop;
 	gs.offset = 0; /* beginning of file */
 	gs.extent = 0;
+	gs.flags = 0;
 	gs.base = NULL;
+	gs.pos = gs.base;
 
 	{
 		/*
@@ -1140,15 +1255,15 @@ nc_get_NC(NC *ncp)
 			extent = ncp->chunk;
 			/* Protection for when ncp->chunk is huge;
 			 * no need to read hugely. */
-	      		if(extent > 8192)
-				extent = 8192;
+	      		if(extent > 4096)
+				extent = 4096;
 		}
 		else if(extent > ncp->chunk)
 		{
 			extent = ncp->chunk;
 		}
 		
-		status = fault_v1hgs(&gs, extent);
+		status = fault_v1hs(&gs, extent);
 		if(status)
 			return status;
 	}
@@ -1160,7 +1275,8 @@ nc_get_NC(NC *ncp)
 		schar magic[sizeof(ncmagic)];
 		(void) memset(magic, 0, sizeof(magic));
 
-		status = ncx_getn_schar_schar(&gs.pos, sizeof(magic), magic);
+		status = ncx_getn_schar_schar(
+			(const void **)(&gs.pos), sizeof(magic), magic);
 		if(status != ENOERR)
 			goto unwind_get;
 	
@@ -1171,7 +1287,7 @@ nc_get_NC(NC *ncp)
 		}
 	}
 	
-	status = ncx_get_size_t(&gs.pos, &ncp->numrecs);
+	status = ncx_get_size_t((const void **)(&gs.pos), &ncp->numrecs);
 	if(status != ENOERR)
 		goto unwind_get;
 
@@ -1194,6 +1310,6 @@ nc_get_NC(NC *ncp)
 	status = NC_computeshapes(ncp);
 
 unwind_get:
-	(void) rel_v1hgs(&gs);
+	(void) rel_v1hs(&gs);
 	return status;
 }
