@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1996, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: test_write.c,v 1.18 1997/06/11 19:37:06 steve Exp $
+ *   $Id: test_write.c,v 1.19 1997/10/17 18:21:44 davis Exp $
  *********************************************************************/
 
 #include "tests.h"
@@ -84,8 +84,11 @@ void
 test_nc_redef(void)
 {
     int ncid;                   /* netcdf id */
+    /* used to force effective test of ncio->move() in redef */
+    size_t sizehint = 8192;
     int dimid;         /* dimension id */
     int varid;         /* variable id */
+    int varid1;        /* variable id */
     int err;
     char * title = "Not funny";
     double var;
@@ -115,11 +118,14 @@ test_nc_redef(void)
 	error("nc_close: %s", nc_strerror(err));
 
 	/* tests using scratch file */
-    err = nc_create(scratch, NC_NOCLOBBER, &ncid);
+    err = nc__create(scratch, NC_NOCLOBBER, 0, &sizehint, &ncid);
     IF (err) {
         error("nc_create: %s", nc_strerror(err));
         return;
     }
+    /* limit for ncio implementations which which have infinite chunksize */
+    if(sizehint > 32768)
+	sizehint = 16384;
     def_dims(ncid);
     def_vars(ncid);
     put_atts(ncid);
@@ -137,18 +143,36 @@ test_nc_redef(void)
     IF (err)
         error("nc_enddef: %s", nc_strerror(err));
     put_vars(ncid);
-    err = nc_def_dim(ncid, "abc", 8, &dimid);
+    err = nc_def_dim(ncid, "abc", sizehint, &dimid);
     IF (err != NC_ENOTINDEFINE)
         error("nc_def_dim in define mode: status = %d", err);
     err = nc_redef(ncid);
     IF (err)
         error("nc_redef: %s", nc_strerror(err));
-    err = nc_def_dim(ncid, "abc", 8, &dimid);
+#if 0
+    err = nc_set_fill(ncid, NC_NOFILL, NULL);
+    IF (err)
+        error("nc_set_fill: %s", nc_strerror(err));
+#endif
+    err = nc_def_dim(ncid, "abc", sizehint, &dimid);
     IF (err)
         error("nc_def_dim: %s", nc_strerror(err));
-    err = nc_def_var(ncid, "abc", NC_INT, 0, NULL, &varid);
+    err = nc_def_var(ncid, "abcScalar", NC_INT, 0, NULL, &varid);
     IF (err)
         error("nc_def_var: %s", nc_strerror(err));
+    err = nc_def_var(ncid, "abc", NC_INT, 1, &dimid, &varid1);
+    IF (err)
+        error("nc_def_var: %s", nc_strerror(err));
+    {
+	int dimids[NDIMS +1];
+	int ii = 0;
+	for(ii = 0; ii < NDIMS; ii++)
+		dimids[ii] = ii;
+	dimids[NDIMS] = dimid;
+    	err = nc_def_var(ncid, "abcRec", NC_INT, NDIMS, dimids, &varid1);
+    	IF (err)
+        	error("nc_def_var: %s", nc_strerror(err));
+    }
     err = nc_put_att_text(ncid, NC_GLOBAL, "title", 1+strlen(title), title);
     IF (err)
         error("nc_put_att_text: %s", nc_strerror(err));
@@ -173,7 +197,7 @@ test_nc_redef(void)
 	error("nc_inq_dim: %s", nc_strerror(err));
     IF (strcmp(name, "abc") != 0) 
 	error("Unexpected dim name");
-    IF (length != 8) 
+    IF (length != sizehint) 
 	error("Unexpected dim length");
     err = nc_get_var1_double(ncid, varid, NULL, &var);
     IF (err)
