@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1996, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: util.c,v 1.22 1997/05/23 18:45:34 steve Exp $
+ *   $Id: util.c,v 1.23 1997/10/17 18:10:57 davis Exp $
  *********************************************************************/
 
 #include <math.h>
@@ -736,8 +736,14 @@ check_vars(int  ncid)
 		IF (err)
 		    error("nc_get_var1_text: %s", nc_strerror(err));
 		IF (text != expect) {
-		    error("Var value read not that expected");
+		    error("Var %s value read 0x%02x not that expected 0x%02x ",
+			var_name[i], text, (char)expect);
+			print_n_size_t(var_rank[i], index);
 		} else {
+#if 0
+			print("\nOk %s ", var_name[i]);
+			print_n_size_t(var_rank[i], index);
+#endif
 		    nok++;
 		}
 	    } else {
@@ -747,8 +753,14 @@ check_vars(int  ncid)
 			error("nc_get_var1_double: %s", nc_strerror(err));
 		    } else {
 			IF (!equal(value,expect,var_type[i], NCT_DOUBLE)) {
-			    error("Var value read not that expected");
+	error("Var %s value read % 12.5e not that expected % 12.7e ",
+		var_name[i], value, expect);
+		print_n_size_t(var_rank[i], index);
 			} else {
+#if 0
+			print("\nOk %s ", var_name[i]);
+			print_n_size_t(var_rank[i], index);
+#endif
 			    nok++;
 			}
 		    }
