@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, University Corporation for Atmospheric Research
  *   See netcdf/README file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/ncdump/ncdump.c,v 1.6 1997/05/19 20:49:03 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncdump/ncdump.c,v 1.8 1997/09/02 22:14:35 russ Exp $
  *********************************************************************/
 
 #include <stdio.h>
@@ -50,6 +50,10 @@ usage(void)
 		   "%s [-c|-h] [-v ...] [[-b|-f] [c|f]] [-l len] [-n name] [-p n[,n]] file\n%s",
 		   progname,
 		   USAGE);
+    
+    (void) fprintf(stderr,
+                 "netcdf library version %s\n",
+                 nc_inq_libvers());
     exit(EXIT_FAILURE);
 }
 
@@ -214,7 +218,7 @@ pr_att_vals(
      )
 {
     int iel;
-    unsigned char uc;
+    signed char sc;
     short ss;
     int ii;
     char gps[30];
@@ -226,11 +230,8 @@ pr_att_vals(
     for (iel = 0; iel < len-1; iel++) {
 	switch (type) {
 	case NC_BYTE:
-	    uc = (unsigned char) vals[iel] & 0377;
-	    if (isprint(uc))
-		Printf ("'%c', ", uc);
-	    else
-		Printf ("'\\%o', ", uc);
+	    sc = (signed char) vals[iel] & 0377;
+	    Printf ("%db, ", sc);
 	    break;
 	case NC_SHORT:
 	    ss = vals[iel];
@@ -258,11 +259,8 @@ pr_att_vals(
     }
     switch (type) {
     case NC_BYTE:
-	uc = (unsigned char) vals[iel] & 0377;
-	if (isprint(uc))
-	    Printf ("'%c'", uc);
-	else
-	    Printf ("'\\%o'", uc);
+	sc = (signed char) vals[iel] & 0377;
+	Printf ("%db", sc);
 	break;
     case NC_SHORT:
 	ss = vals[iel];
