@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/ncgen/genlib.c,v 1.29 1997/06/12 20:29:48 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncgen/genlib.c,v 1.36 1998/03/03 00:52:17 davis Exp $
  *********************************************************************/
 
 #include <stdio.h>
@@ -226,7 +226,7 @@ gen_c(
 	cline("");
 	cline("   /* dimension ids */");
 	for (idim = 0; idim < ndims; idim++) {
-	    sprintf(stmnt, "   int %s_dim;", dims[idim].name);
+	    sprintf(stmnt, "   int %s_dim;", dims[idim].lname);
 	    cline(stmnt);
 	    }
 
@@ -235,11 +235,11 @@ gen_c(
 	for (idim = 0; idim < ndims; idim++) {
 	    if (dims[idim].size == NC_UNLIMITED) {
 		sprintf(stmnt, "   size_t %s_len = NC_UNLIMITED;",
-			dims[idim].name);
+			dims[idim].lname);
 	    } else {
-		sprintf(stmnt, "   size_t %s_len = %d;",
-			dims[idim].name,
-			dims[idim].size);
+		sprintf(stmnt, "   size_t %s_len = %lu;",
+			dims[idim].lname,
+			(unsigned long) dims[idim].size);
 	    }
 	    cline(stmnt);
 	}
@@ -254,25 +254,24 @@ gen_c(
 	cline("");
 	cline("   /* variable ids */");
 	for (ivar = 0; ivar < nvars; ivar++) {
-	    sprintf(stmnt, "   int %s_id;", vars[ivar].name);
+	    sprintf(stmnt, "   int %s_id;", vars[ivar].lname);
 	    cline(stmnt);
 	}
 
+	cline("");
+	cline("   /* rank (number of dimensions) for each variable */");
+	for (ivar = 0; ivar < nvars; ivar++) {
+	    sprintf(stmnt, "#  define RANK_%s %d", vars[ivar].lname,
+		    vars[ivar].ndims);
+	    cline(stmnt);
+	}
 	if (maxdims > 0) {	/* we have dimensioned variables */
-	    cline("");
-	    cline("   /* rank (number of dimensions) for each variable */");
-	    for (ivar = 0; ivar < nvars; ivar++) {
-		sprintf(stmnt, "#  define RANK_%s %d", vars[ivar].name,
-			vars[ivar].ndims);
-		cline(stmnt);
-	    }
-
 	    cline("");
 	    cline("   /* variable shapes */");
 	    for (ivar = 0; ivar < nvars; ivar++) {
 		if (vars[ivar].ndims > 0) {
-		    sprintf(stmnt, "   int %s_dims[RANK_%s];", vars[ivar].name,
-			    vars[ivar].name);
+		    sprintf(stmnt, "   int %s_dims[RANK_%s];",
+			    vars[ivar].lname, vars[ivar].lname);
 		    cline(stmnt);
 		}
 	    }
@@ -293,11 +292,11 @@ gen_c(
 	for (iatt = 0; iatt < natts; iatt++) {
 	    if (atts[iatt].type != NC_CHAR) {
 		sprintf(stmnt,
-		    "   %s %s_%s[%d];",
+		    "   %s %s_%s[%lu];",
 		    ncatype(atts[iatt].type),
-		    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
-		    atts[iatt].name,
-		    atts[iatt].len);
+		    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].lname,
+		    atts[iatt].lname,
+		    (unsigned long) atts[iatt].len);
 		cline(stmnt);
 	    }
 	}
@@ -320,7 +319,7 @@ gen_c(
     for (idim = 0; idim < ndims; idim++) {
 	sprintf(stmnt,
 		"   stat = nc_def_dim(ncid, \"%s\", %s_len, &%s_dim);",
-		dims[idim].name, dims[idim].name, dims[idim].name);
+		dims[idim].name, dims[idim].lname, dims[idim].lname);
 	cline(stmnt);
 	cline("   check_err(stat,__LINE__,__FILE__);");
     }
@@ -334,9 +333,9 @@ gen_c(
 	    for (idim = 0; idim < vars[ivar].ndims; idim++) {
 		sprintf(stmnt,
 			"   %s_dims[%d] = %s_dim;",
-			vars[ivar].name,
+			vars[ivar].lname,
 			idim,
-			dims[vars[ivar].dims[idim]].name);
+			dims[vars[ivar].dims[idim]].lname);
 		cline(stmnt);
 	    }
 	    if (vars[ivar].ndims > 0) {	/* a dimensioned variable */
@@ -344,16 +343,16 @@ gen_c(
 			"   stat = nc_def_var(ncid, \"%s\", %s, RANK_%s, %s_dims, &%s_id);",
 			vars[ivar].name,
 			nctype(vars[ivar].type),
-			vars[ivar].name,
-			vars[ivar].name,
-			vars[ivar].name);
+			vars[ivar].lname,
+			vars[ivar].lname,
+			vars[ivar].lname);
 	    } else {		/* a scalar */
 		sprintf(stmnt,
 			"   stat = nc_def_var(ncid, \"%s\", %s, RANK_%s, 0, &%s_id);",
 			vars[ivar].name,
 			nctype(vars[ivar].type),
-			vars[ivar].name,
-			vars[ivar].name);
+			vars[ivar].lname,
+			vars[ivar].lname);
 	    }
 	    cline(stmnt);
 	    cline("   check_err(stat,__LINE__,__FILE__);");
@@ -368,11 +367,11 @@ gen_c(
 	    if (atts[iatt].type == NC_CHAR) { /* string */
 		val_string = cstrstr((char *) atts[iatt].val, atts[iatt].len);
 		sprintf(stmnt,
-			"   stat = nc_put_att_text(ncid, %s%s, \"%s\", %d, %s);",
-			atts[iatt].var == -1 ? "NC_GLOBAL" : vars[atts[iatt].var].name,
+			"   stat = nc_put_att_text(ncid, %s%s, \"%s\", %lu, %s);",
+			atts[iatt].var == -1 ? "NC_GLOBAL" : vars[atts[iatt].var].lname,
 			atts[iatt].var == -1 ? "" : "_id",
 			atts[iatt].name,
-			atts[iatt].len,
+			(unsigned long) atts[iatt].len,
 			val_string);
 		cline(stmnt);
 		free (val_string);
@@ -381,8 +380,8 @@ gen_c(
 		for (jatt = 0; jatt < atts[iatt].len ; jatt++) {
 		    val_string = cstring(atts[iatt].type,atts[iatt].val,jatt);
 		    sprintf(stmnt, "   %s_%s[%d] = %s;",
-			    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
-			    atts[iatt].name,
+			    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].lname,
+			    atts[iatt].lname,
 			    jatt, 
 			    val_string);
 		    cline(stmnt);
@@ -390,15 +389,15 @@ gen_c(
 		}
 		
 		sprintf(stmnt,
-			"   stat = nc_put_att_%s(ncid, %s%s, \"%s\", %s, %d, %s_%s);",
+			"   stat = nc_put_att_%s(ncid, %s%s, \"%s\", %s, %lu, %s_%s);",
 			ncatype(atts[iatt].type),
-			atts[iatt].var == -1 ? "NC_GLOBAL" : vars[atts[iatt].var].name,
+			atts[iatt].var == -1 ? "NC_GLOBAL" : vars[atts[iatt].var].lname,
 			atts[iatt].var == -1 ? "" : "_id",
 			atts[iatt].name,
 			nctype(atts[iatt].type),
-			atts[iatt].len,
-			atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
-			atts[iatt].name);
+			(unsigned long) atts[iatt].len,
+			atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].lname,
+			atts[iatt].lname);
 		cline(stmnt);
 	    }
 	    cline("   check_err(stat,__LINE__,__FILE__);");
@@ -570,23 +569,23 @@ gen_fortran(
     if (ndims > 0) {
 	fline("* dimension ids");
 	for (idim = 0; idim < ndims; idim++) {
-	    sprintf(stmnt, "integer  %s_dim", dims[idim].name);
+	    sprintf(stmnt, "integer  %s_dim", dims[idim].lname);
 	    fline(stmnt);
 	}
 
 	fline("* dimension lengths");
 	for (idim = 0; idim < ndims; idim++) {
-	    sprintf(stmnt, "integer  %s_len", dims[idim].name);
+	    sprintf(stmnt, "integer  %s_len", dims[idim].lname);
 	    fline(stmnt);
 	}
 	for (idim = 0; idim < ndims; idim++) {
 	    if (dims[idim].size == NC_UNLIMITED) {
 		sprintf(stmnt, "parameter (%s_len = NF_UNLIMITED)",
-			dims[idim].name);
+			dims[idim].lname);
 	    } else {
-		sprintf(stmnt, "parameter (%s_len = %d)",
-			dims[idim].name,
-			dims[idim].size);
+		sprintf(stmnt, "parameter (%s_len = %lu)",
+			dims[idim].lname,
+			(unsigned long) dims[idim].size);
 	    }
 	    fline(stmnt);
 	}
@@ -601,17 +600,17 @@ gen_fortran(
     if (nvars > 0) {
 	fline("* variable ids");
 	for (ivar = 0; ivar < nvars; ivar++) {
-	    sprintf(stmnt, "integer  %s_id", vars[ivar].name);
+	    sprintf(stmnt, "integer  %s_id", vars[ivar].lname);
 	    fline(stmnt);
 	}
 
 	fline("* rank (number of dimensions) for each variable");
 	for (ivar = 0; ivar < nvars; ivar++) {
-	    sprintf(stmnt, "integer  %s_rank", vars[ivar].name);
+	    sprintf(stmnt, "integer  %s_rank", vars[ivar].lname);
 	    fline(stmnt);
 	}
 	for (ivar = 0; ivar < nvars; ivar++) {
-	    sprintf(stmnt, "parameter (%s_rank = %d)", vars[ivar].name,
+	    sprintf(stmnt, "parameter (%s_rank = %d)", vars[ivar].lname,
 		    vars[ivar].ndims);
 	    fline(stmnt);
 	}
@@ -620,7 +619,7 @@ gen_fortran(
 	for (ivar = 0; ivar < nvars; ivar++) {
 	    if (vars[ivar].ndims > 0) {
 		sprintf(stmnt, "integer  %s_dims(%s_rank)",
-			vars[ivar].name, vars[ivar].name);
+			vars[ivar].lname, vars[ivar].lname);
 		fline(stmnt);
 	    }
 	}
@@ -644,14 +643,14 @@ gen_fortran(
             }
 	    if (v->ndims == 0) { /* scalar */
 		sprintf(stmnt, "%s  %s", ncftype(v->type),
-			v->name);
+			v->lname);
 	    } else {
 		sprintf(stmnt, "%s  %s(", ncftype(v->type),
-			v->name);
+			v->lname);
 		/* reverse dimensions for FORTRAN */
 		for (idim = v->ndims-1; idim >= 0; idim--) {
 		    sprintf(s2, "%s_len, ",
-			    dims[v->dims[idim]].name);
+			    dims[v->dims[idim]].lname);
 		    strcat(stmnt, s2);
 		}
 		sp = strrchr(stmnt, ',');
@@ -679,8 +678,9 @@ gen_fortran(
 	fline("* attribute vectors");
 	for (itype = 0; itype < ntypes; itype++) {
 	    if (types[itype] != NC_CHAR && max_atts[(int)types[itype]] > 0) {
-		sprintf(stmnt, "%s  %sval(%d)", ncftype(types[itype]),
-			nfstype(types[itype]), max_atts[(int)types[itype]]);
+		sprintf(stmnt, "%s  %sval(%lu)", ncftype(types[itype]),
+			nfstype(types[itype]),
+			(unsigned long) max_atts[(int)types[itype]]);
 		fline(stmnt);
 	    }
 	}
@@ -698,10 +698,11 @@ gen_fortran(
     for (idim = 0; idim < ndims; idim++) {
 	if (dims[idim].size == NC_UNLIMITED)
             sprintf(stmnt, "iret = nf_def_dim(ncid, \'%s\', NF_UNLIMITED, %s_dim)",
-                    dims[idim].name, dims[idim].name);
+                    dims[idim].name, dims[idim].lname);
 	else
-            sprintf(stmnt, "iret = nf_def_dim(ncid, \'%s\', %d, %s_dim)",
-                    dims[idim].name, dims[idim].size, dims[idim].name);
+            sprintf(stmnt, "iret = nf_def_dim(ncid, \'%s\', %lu, %s_dim)",
+                    dims[idim].name, (unsigned long) dims[idim].size,
+			dims[idim].lname);
 	fline(stmnt);
 	fline("call check_err(iret)");
     }
@@ -712,9 +713,9 @@ gen_fortran(
 	for (ivar = 0; ivar < nvars; ivar++) {
 	    for (idim = 0; idim < vars[ivar].ndims; idim++) {
 		sprintf(stmnt, "%s_dims(%d) = %s_dim",
-			vars[ivar].name,
+			vars[ivar].lname,
 			vars[ivar].ndims - idim, /* reverse dimensions */
-			dims[vars[ivar].dims[idim]].name);
+			dims[vars[ivar].dims[idim]].lname);
 		fline(stmnt);
 	    }
 	    if (vars[ivar].ndims > 0) {	/* a dimensioned variable */
@@ -722,16 +723,16 @@ gen_fortran(
 			"iret = nf_def_var(ncid, \'%s\', %s, %s_rank, %s_dims, %s_id)",
 			vars[ivar].name,
 			ftypename(vars[ivar].type),
-			vars[ivar].name,
-			vars[ivar].name,
-			vars[ivar].name);
+			vars[ivar].lname,
+			vars[ivar].lname,
+			vars[ivar].lname);
 	    } else {		/* a scalar */
 		sprintf(stmnt, 
 			"iret = nf_def_var(ncid, \'%s\', %s, %s_rank, 0, %s_id)",
 			vars[ivar].name,
 			ftypename(vars[ivar].type),
-			vars[ivar].name,
-			vars[ivar].name);
+			vars[ivar].lname,
+			vars[ivar].lname);
 	    }
 	    fline(stmnt);
 	    fline("call check_err(iret)");
@@ -745,11 +746,11 @@ gen_fortran(
 	    if (atts[iatt].type == NC_CHAR) { /* string */
 		val_string = fstrstr((char *) atts[iatt].val, atts[iatt].len);
 		sprintf(stmnt, 
-			"iret = nf_put_att_text(ncid, %s%s, \'%s\', %d, %s)",
-			atts[iatt].var == -1 ? "NF_GLOBAL" : vars[atts[iatt].var].name,
+			"iret = nf_put_att_text(ncid, %s%s, \'%s\', %lu, %s)",
+			atts[iatt].var == -1 ? "NF_GLOBAL" : vars[atts[iatt].var].lname,
 			atts[iatt].var == -1 ? "" : "_id",
 			atts[iatt].name,
-			atts[iatt].len,
+			(unsigned long) atts[iatt].len,
 			val_string);
 		fline(stmnt);
 		fline("call check_err(iret)");
@@ -766,13 +767,13 @@ gen_fortran(
 		}
 	    
 		sprintf(stmnt,
-			"iret = nf_put_att_%s(ncid, %s%s, \'%s\', %s, %d, %sval)",
+			"iret = nf_put_att_%s(ncid, %s%s, \'%s\', %s, %lu, %sval)",
 			nfftype(atts[iatt].type),
-			atts[iatt].var == -1 ? "NCGLOBAL" : vars[atts[iatt].var].name,
+			atts[iatt].var == -1 ? "NCGLOBAL" : vars[atts[iatt].var].lname,
 			atts[iatt].var == -1 ? "" : "_id",
 			atts[iatt].name,
 			ftypename(atts[iatt].type),
-			atts[iatt].len,
+			(unsigned long) atts[iatt].len,
 			nfstype(atts[iatt].type));
 		fline(stmnt);
 		fline("call check_err(iret)");
@@ -1037,6 +1038,7 @@ cstrstr(
     static char *sp;
     char *cp;
     char *istr, *istr0;		/* for null-terminated copy */
+    int ii;
     
     if(4*len+3 != (unsigned)(4*len+3)) {
 	derror("too much character data!");
@@ -1050,12 +1052,15 @@ cstrstr(
     }
 
     istr0 = istr = (char *) emalloc(len + 1);
-    strncpy(istr, valp, len);
+    for(ii = 0; ii < len; ii++) {
+	istr[ii] = valp[ii];
+    }
     istr[len] = '\0';
 
     *cp++ = '"';
-    while (*istr != '\0') {
+    for(ii = 0; ii < len; ii++) {
 	switch (*istr) {
+	case '\0': *cp++ = '\\'; *cp++ = '0'; *cp++ = '0'; *cp++ = '0'; break;
 	case '\b': *cp++ = '\\'; *cp++ = 'b'; break;
 	case '\f': *cp++ = '\\'; *cp++ = 'f'; break;
 	case '\n': *cp++ = '\\'; *cp++ = 'n'; break;
@@ -1103,14 +1108,16 @@ fstrstr(
     char *cp, tstr[12];
     int was_print = 0;		/* true if last character was printable */
     char *istr, *istr0;		/* for null-terminated copy */
-
+    int ii;
 
     if(12*ilen != (size_t)(12*ilen)) {
 	derror("too much character data!");
 	exit(9);
     }
     istr0 = istr = (char *) emalloc(ilen + 1);
-    strncpy(istr, str, ilen);
+    for(ii = 0; ii < ilen; ii++) {
+	istr[ii] = str[ii];
+    }
     istr[ilen] = '\0';
     
     if (*istr == '\0') {	/* empty string input, not legal in FORTRAN */
@@ -1146,7 +1153,7 @@ fstrstr(
     }
     istr++;
 
-    while (*istr != '\0') {	/* handle subsequent characters in input */
+    for(ii = 1; ii < ilen; ii++) { /* handle subsequent characters in input */
 	if (isprint(*istr)) {
 	    if (! was_print) {
 		strcat(cp, "//'");
@@ -1285,7 +1292,7 @@ cl_fortran(void)
             struct vars *v = &vars[ivar];
             /* if a record variable, include id in parameter list */
             if (v->ndims > 0 && v->dims[0] == rec_dim) {
-                sprintf(s2, "%s_id,", v->name);
+                sprintf(s2, "%s_id,", v->lname);
                 strcat(stmnt, s2);
             }
         }        
@@ -1309,7 +1316,7 @@ cl_fortran(void)
         for (ivar = 0; ivar < nvars; ivar++) {
             struct vars *v = &vars[ivar];
             if (v->ndims > 0 && v->dims[0] == rec_dim) {
-                sprintf(s2, "%s_id,", v->name);
+                sprintf(s2, "%s_id,", v->lname);
                 strcat(stmnt, s2);
             }
         }        
@@ -1327,7 +1334,7 @@ cl_fortran(void)
 	for (ivar = 0; ivar < nvars; ivar++) {
 	    struct vars *v = &vars[ivar];
             if (v->ndims > 0 && v->dims[0] == rec_dim) {
-                sprintf(stmnt, "integer  %s_id", v->name);
+                sprintf(stmnt, "integer  %s_id", v->lname);
                 fline(stmnt);
             }
 	}
@@ -1346,10 +1353,10 @@ cl_fortran(void)
         for (idim = 0; idim < ndims; idim++) {
             /* if used in a record variable and not record dimension */
             if (used_in_rec_var(idim) && dims[idim].size != NC_UNLIMITED) {
-                sprintf(stmnt, "integer  %s_len", dims[idim].name);
+                sprintf(stmnt, "integer  %s_len", dims[idim].lname);
                 fline(stmnt);
-                sprintf(stmnt, "parameter (%s_len = %d)",
-                        dims[idim].name, dims[idim].size);
+                sprintf(stmnt, "parameter (%s_len = %lu)",
+                        dims[idim].lname, (unsigned long) dims[idim].size);
                 fline(stmnt);
             }
         }
@@ -1359,14 +1366,14 @@ cl_fortran(void)
 	for (ivar = 0; ivar < nvars; ivar++) {
 	    struct vars *v = &vars[ivar];
             if (v->ndims > 0 && v->dims[0] == rec_dim) {
-                sprintf(stmnt, "integer  %s_rank", v->name);
+                sprintf(stmnt, "integer  %s_rank", v->lname);
                 fline(stmnt);
             }
 	}
 	for (ivar = 0; ivar < nvars; ivar++) {
 	    struct vars *v = &vars[ivar];
             if (v->ndims > 0 && v->dims[0] == rec_dim) {
-                sprintf(stmnt, "parameter (%s_rank = %d)", v->name,
+                sprintf(stmnt, "parameter (%s_rank = %d)", v->lname,
                         v->ndims);
                 fline(stmnt);
             }
@@ -1378,7 +1385,7 @@ cl_fortran(void)
 	    if (v->ndims > 0 && v->dims[0] == rec_dim) {
 		sprintf(stmnt,
 			"integer  %s_start(%s_rank), %s_count(%s_rank)",
-			v->name, v->name, v->name, v->name);
+			v->lname, v->lname, v->lname, v->lname);
 		fline(stmnt);
 	    }
 	}
@@ -1392,26 +1399,26 @@ cl_fortran(void)
                 char *sp;
 	    
                 fline(" ");
-                sprintf(stmnt, "integer  %s_nr", v->name);
+                sprintf(stmnt, "integer  %s_nr", v->lname);
                 fline(stmnt);
                 if (v->nrecs > 0) {
-                    sprintf(stmnt, "parameter (%s_nr = %d)",
-                            v->name, v->nrecs);
+                    sprintf(stmnt, "parameter (%s_nr = %lu)",
+                            v->lname, (unsigned long) v->nrecs);
                 } else {
-                    sprintf(stmnt, "parameter (%s_nr = 1)  ! really 0, so use a record of fill data",
-                            v->name);
+                    sprintf(stmnt, "parameter (%s_nr = 1)",
+                            v->lname);
                 }
                 fline(stmnt);
 		if (v->type != NC_CHAR) {
 		    sprintf(stmnt, "%s  %s(", ncftype(v->type),
-			    v->name);
+			    v->lname);
 		    /* reverse dimensions for FORTRAN */
 		    for (idim = v->ndims-1; idim >= 0; idim--) {
 			if(v->dims[idim] == rec_dim) {
-			    sprintf(s2, "%s_nr, ", v->name);
+			    sprintf(s2, "%s_nr, ", v->lname);
 			} else {
 			    sprintf(s2, "%s_len, ",
-				    dims[v->dims[idim]].name);
+				    dims[v->dims[idim]].lname);
 			}
 			strcat(stmnt, s2);
 		    }
@@ -1440,7 +1447,8 @@ cl_fortran(void)
                     for (idim = 1; idim < v->ndims; idim++) {
                         rec_len *= dims[v->dims[idim]].size;
                     }
-                    sprintf(stmnt,"data %s /%d * %s/", v->name, rec_len,
+                    sprintf(stmnt,"data %s /%lu * %s/", v->lname,
+			(unsigned long) rec_len,
                             f_fill_name(v->type));		
                     fline(stmnt);
                 }
@@ -1457,26 +1465,26 @@ cl_fortran(void)
 		fline(stmnt);
 
 		for (idim = 0; idim < v->ndims; idim++) {
-		    sprintf(stmnt, "%s_start(%d) = 1", v->name, idim+1);
+		    sprintf(stmnt, "%s_start(%d) = 1", v->lname, idim+1);
 		    fline(stmnt);
 		}
 		for (idim = v->ndims-1; idim > 0; idim--) {
-		    sprintf(stmnt, "%s_count(%d) = %s_len", v->name,
-			    v->ndims - idim, dims[v->dims[idim]].name);
+		    sprintf(stmnt, "%s_count(%d) = %s_len", v->lname,
+			    v->ndims - idim, dims[v->dims[idim]].lname);
 		    fline(stmnt);
 		}
-                sprintf(stmnt, "%s_count(%d) = %s_nr", v->name,
-                        v->ndims, v->name);
+                sprintf(stmnt, "%s_count(%d) = %s_nr", v->lname,
+                        v->ndims, v->lname);
 		fline(stmnt);
 		
 		if (v->type != NC_CHAR) {
 		    sprintf(stmnt,
 			    "iret = nf_put_vara_%s(ncid, %s_id, %s_start, %s_count, %s)",
-			    nfftype(v->type), v->name, v->name, v->name, v->name);
+			    nfftype(v->type), v->lname, v->lname, v->lname, v->lname);
 		} else {
 		    sprintf(stmnt,
 			    "iret = nf_put_vara_%s(ncid, %s_id, %s_start, %s_count, %s)",
-			    nfftype(v->type), v->name, v->name, v->name,
+			    nfftype(v->type), v->lname, v->lname, v->lname,
 			    v->data_stmnt);
 		}
 		
@@ -1752,3 +1760,53 @@ grow_aarray(
     return;
   *arpp = (struct atts *) erealloc(*arpp, 2 * nar * sizeof(struct atts));
 }
+
+
+/*
+ * Replace dashes and dots in name so it can be used in C and
+ * Fortran without causing syntax errors.  Here we just replace each "-"
+ * in a name with "_dash_" and each "." with "_dot_", though any
+ * similar replacement that doesn't clash with existing names would
+ * work.
+ */
+extern char*
+decodify (
+    const char *name)
+{
+    int count=0;		/* number of minus signs in name */
+    char *newname;
+    const char *cp = name;
+    char *sp;
+
+    while(*cp != '\0') {
+	if (*cp == '-')
+	    count += 5;
+	else if(*cp == '.')
+	    count += 4;
+	cp++;
+    }
+    newname = (char *) emalloc(strlen(name) + count + 1);
+    cp = name;
+    sp = newname;
+    while(*cp != '\0') {
+	if (*cp == '-') {
+	    *sp++ = '_';
+	    *sp++ = 'd';
+	    *sp++ = 'a';
+	    *sp++ = 's';
+	    *sp++ = 'h';
+	    *sp++ = '_';
+	} else if (*cp == '.') {
+	    *sp++ = '_';
+	    *sp++ = 'd';
+	    *sp++ = 'o';
+	    *sp++ = 't';
+	    *sp++ = '_';
+	} else {
+	    *sp++ = *cp;
+	}
+	cp++;
+    }
+    *sp = '\0';
+    return newname;
+}
