@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: load.c,v 1.27 1997/06/02 14:57:42 russ Exp $
+ *   $Id: load.c,v 1.29 1997/07/07 18:27:03 russ Exp $
  *********************************************************************/
 
 #include <stdio.h>
@@ -79,32 +79,18 @@ gen_load_c(
     if (vars[varnum].ndims > 0) {
 	if (vars[varnum].dims[0] == rec_dim) {
 	    sprintf(stmnt, "    static size_t %s_start[RANK_%s];",
-		    vars[varnum].name, vars[varnum].name);
+		    vars[varnum].lname, vars[varnum].lname);
 	    cline(stmnt);
 
 	    sprintf(stmnt, "    static size_t %s_count[RANK_%s];",
-		    vars[varnum].name, vars[varnum].name);
+		    vars[varnum].lname, vars[varnum].lname);
 	    cline(stmnt);
-
-	    for (idim = 0; idim < vars[varnum].ndims; idim++) {
-		sprintf(s2, "%d%s",
-			0,
-			idim < vars[varnum].ndims-1 ? ", " : "};");
-		strncat(stmnt, s2, C_MAX_STMNT - strlen(stmnt) );
-	    }
-	
-	    for (idim = 0; idim < vars[varnum].ndims; idim++) {
-		sprintf(s2, "%ld%s",
-			dims[vars[varnum].dims[idim]].size,
-			idim < vars[varnum].ndims-1 ? ", " : "};");
-		strncat(stmnt, s2, C_MAX_STMNT - strlen(stmnt) );
-	    }
 	}
 	
 	/* load variable with data values using static initialization */
 	sprintf(stmnt, "    static %s %s[] = {",
 		ncctype(vars[varnum].type),
-		vars[varnum].name);
+		vars[varnum].lname);
 	
 	stmnt_len = strlen(stmnt);
 	switch (vars[varnum].type) {
@@ -197,23 +183,23 @@ gen_load_c(
 	if (vars[varnum].dims[0] == rec_dim) {
 	    sprintf(stmnt,
 		    "    %s_len = %d;			/* number of records of %s data */",
-		    dims[rec_dim].name,
+		    dims[rec_dim].lname,
 		    vars[varnum].nrecs, /* number of recs for this variable */
 		    vars[varnum].name);
 	    cline(stmnt);
 	    
 	    for (idim = 0; idim < vars[varnum].ndims; idim++) {
 		sprintf(stmnt, "    %s_start[%d] = 0;",
-			vars[varnum].name,
+			vars[varnum].lname,
 			idim);
 		cline(stmnt);
 	    }
 
 	    for (idim = 0; idim < vars[varnum].ndims; idim++) {
 		sprintf(stmnt, "    %s_count[%d] = %s_len;",
-			vars[varnum].name,
+			vars[varnum].lname,
 			idim,
-			dims[vars[varnum].dims[idim]].name);
+			dims[vars[varnum].dims[idim]].lname);
 		cline(stmnt);
 	    }
 	}
@@ -222,23 +208,23 @@ gen_load_c(
 	    sprintf(stmnt,
 		    "    stat = nc_put_vara_%s(ncid, %s_id, %s_start, %s_count, %s);",
 		    ncstype(vars[varnum].type),
-		    vars[varnum].name,
-		    vars[varnum].name,
-		    vars[varnum].name,
-		    vars[varnum].name);
+		    vars[varnum].lname,
+		    vars[varnum].lname,
+		    vars[varnum].lname,
+		    vars[varnum].lname);
 	} else {		/* non-record variables */
 	    sprintf(stmnt,
 		    "    stat = nc_put_var_%s(ncid, %s_id, %s);",
 		    ncstype(vars[varnum].type),
-		    vars[varnum].name,
-		    vars[varnum].name);
+		    vars[varnum].lname,
+		    vars[varnum].lname);
 	}
 	cline(stmnt);
     } else {			/* scalar variables */
 	/* load variable with data values using static initialization */
 	sprintf(stmnt, "    static %s %s = ",
 		ncctype(vars[varnum].type),
-		vars[varnum].name);
+		vars[varnum].lname);
 	
 	switch (vars[varnum].type) {
 	  case NC_CHAR:
@@ -275,8 +261,8 @@ gen_load_c(
 	sprintf(stmnt,
 		"    stat = nc_put_var_%s(ncid, %s_id, &%s);",
 		ncstype(vars[varnum].type),
-		vars[varnum].name,
-		vars[varnum].name);
+		vars[varnum].lname,
+		vars[varnum].lname);
 	cline(stmnt);
     }
     cline("    check_err(stat,__LINE__,__FILE__);");
@@ -331,7 +317,7 @@ f_var_init(
     int ival;
     
     /* load variable with data values  */
-    sprintf(stmnt, "data %s /",vars[varnum].name);
+    sprintf(stmnt, "data %s /",vars[varnum].lname);
     stmnt_len = strlen(stmnt);
     switch (vars[varnum].type) {
     case NC_BYTE:
@@ -434,11 +420,11 @@ gen_load_fortran(
     }
     if (v->type != NC_CHAR) {
 	sprintf(stmnt, "iret = nf_put_var_%s(ncid, %s_id, %s)",
-		nfftype(v->type), v->name, v->name);
+		nfftype(v->type), v->lname, v->lname);
     } else {
 	char *char_expr = fstrstr(rec_start, valnum);
 	sprintf(stmnt, "iret = nf_put_var_%s(ncid, %s_id, %s)",
-		nfftype(v->type), v->name, char_expr);
+		nfftype(v->type), v->lname, char_expr);
 	free(char_expr);
     }
     
