@@ -1,7 +1,7 @@
 
 # line 10 "ncgen.y"
 #ifndef lint
-static char SccsId[] = "$Id: ncgen.y,v 1.12 1997/04/29 15:48:25 russ Exp $";
+static char SccsId[] = "$Id: ncgen.y,v 1.19 1997/09/02 22:16:34 russ Exp $";
 #endif
 #include        <string.h>
 #include	<stdlib.h>
@@ -37,7 +37,7 @@ static float float_val;		/* last float value read */
 static int int_val;		/* last int value read */
 static short short_val;		/* last short value read */
 static char char_val;		/* last char value read */
-static char byte_val;		/* last byte value read */
+static signed char byte_val;	/* last byte value read */
 
 static nc_type type_code;	/* holds declared type for variables */
 static nc_type atype_code;	/* holds derived type for attributes */
@@ -46,7 +46,7 @@ static void *att_space;		/* pointer to block for attribute values */
 static nc_type valtype;		/* type code for list of attribute values  */
 
 static char *char_valp;		/* pointers used to accumulate data values */
-static char *byte_valp;
+static signed char *byte_valp;
 static short *short_valp;
 static int *int_valp;
 static float *float_valp;
@@ -123,7 +123,7 @@ YYSTYPE *yyv;
 static int yymaxdepth = YYMAXDEPTH;
 # define YYERRCODE 256
 
-# line 742 "ncgen.y"
+# line 752 "ncgen.y"
 
 
 /* PROGRAMS */
@@ -198,14 +198,14 @@ clearout(void)	/* reset symbol table to empty */
 
 /* get lexical input routine generated by lex  */
 #include "ncgenyy.c"
-yytabelem yyexca[] ={
+static const yytabelem yyexca[] ={
 -1, 1,
 	0, -1,
 	-2, 0,
 	};
 # define YYNPROD 75
 # define YYLAST 232
-yytabelem yyact[]={
+static const yytabelem yyact[]={
 
     97,    98,    96,    99,   100,   101,   102,    46,    14,     6,
      2,   103,    70,    71,    69,    72,    73,    74,    75,    58,
@@ -231,7 +231,7 @@ yytabelem yyact[]={
      0,     0,     0,     0,    16,     0,     0,     0,     0,     0,
      0,     0,     0,     8,    29,    30,    31,    32,    33,    34,
     16,    16 };
-yytabelem yypact[]={
+static const yytabelem yypact[]={
 
   -264,-10000000,   -97,-10000000,  -263,   -50,  -244,-10000000,   -34,   -33,
    -13,-10000000,-10000000,   -11,  -242,-10000000,-10000000,  -244,    -4,-10000000,
@@ -244,13 +244,13 @@ yytabelem yypact[]={
 -10000000,  -244,  -253,-10000000,-10000000,     7,-10000000,-10000000,-10000000,     5,
 -10000000,  -265,-10000000,  -244,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
 -10000000,-10000000,-10000000,-10000000,-10000000,-10000000 };
-yytabelem yypgo[]={
+static const yytabelem yypgo[]={
 
      0,    94,    92,    91,    90,    89,    88,    87,    60,    53,
     86,    29,    85,    82,    59,    80,    58,    62,    79,    77,
     33,    28,    76,    75,    74,    30,    73,    71,    70,    34,
     61,    32,    69,    68,    38,    66,    65,    27,    64,    63 };
-yytabelem yyr1[]={
+static const yytabelem yyr1[]={
 
      0,     2,     5,     1,     3,     3,     7,     7,     8,     8,
      9,     9,    10,    11,     4,     4,     4,    12,    12,    14,
@@ -260,7 +260,7 @@ yytabelem yyr1[]={
     32,    32,    32,    32,    32,    32,    32,     6,     6,    33,
     33,    35,    34,    36,    36,    38,    37,    39,    39,    39,
     39,    39,    39,    39,    39 };
-yytabelem yyr2[]={
+static const yytabelem yyr2[]={
 
      0,     1,     1,    17,     0,     4,     4,     6,     2,     6,
      7,     7,     3,     2,     0,     4,     2,     4,     6,     2,
@@ -270,7 +270,7 @@ yytabelem yyr2[]={
      3,     3,     3,     3,     3,     3,     3,     0,     4,     4,
      6,     1,     9,     2,     6,     1,     5,     3,     3,     3,
      3,     3,     3,     3,     3 };
-yytabelem yychk[]={
+static const yytabelem yychk[]={
 
 -10000000,    -1,   274,   123,    -2,    -3,   272,    -4,   273,   -13,
    -17,   -16,   -26,   -29,    58,   -21,   264,    -7,    -8,    -9,
@@ -283,7 +283,7 @@ yytabelem yychk[]={
    -23,    40,    44,    59,    61,   -24,   -25,   -11,   -31,   -36,
    -37,   -38,    41,    44,    44,   -39,   267,   265,   266,   268,
    269,   270,   271,   276,   -25,   -37 };
-yytabelem yydef[]={
+static const yytabelem yydef[]={
 
      0,    -2,     0,     1,     4,    14,     0,     2,     0,    16,
      0,    23,    41,     0,     0,    45,    35,     5,     0,     8,
@@ -416,7 +416,7 @@ char * yyreds[] =
  * Copyright (c) 1993 by Sun Microsystems, Inc.
  */
 
-#pragma ident	"@(#)yaccpar	6.12	93/06/07 SMI"
+#pragma ident	"@(#)yaccpar	6.14	97/01/16 SMI"
 
 /*
 ** Skeleton parser driver for yacc output
@@ -443,7 +443,7 @@ char * yyreds[] =
 #define YYRECOVERING()	(!!yyerrflag)
 #define YYNEW(type)	malloc(sizeof(type) * yynewmax)
 #define YYCOPY(to, from, type) \
-	(type *) memcpy(to, (char *) from, yynewmax * sizeof(type))
+	(type *) memcpy(to, (char *) from, yymaxdepth * sizeof (type))
 #define YYENLARGE( from, type) \
 	(type *) realloc((char *) from, yynewmax * sizeof(type))
 #ifndef YYDEBUG
@@ -527,12 +527,12 @@ int yyparse(void)
 int yyparse()
 #endif
 {
-	register YYSTYPE *yypvt;	/* top of value stack for $vars */
+	register YYSTYPE *yypvt = 0;	/* top of value stack for $vars */
 
 #if defined(__cplusplus) || defined(lint)
 /*
-	hacks to please C++ and lint - goto's inside switch should never be
-	executed; yypvt is set to 0 to avoid "used before set" warning.
+	hacks to please C++ and lint - goto's inside
+	switch should never be executed
 */
 	static int __yaccpar_lint_hack__ = 0;
 	switch (__yaccpar_lint_hack__)
@@ -540,7 +540,6 @@ int yyparse()
 		case 1: goto yyerrlab;
 		case 2: goto yynewstate;
 	}
-	yypvt = 0;
 #endif
 
 	/*
@@ -760,7 +759,7 @@ int yyparse()
 			** look through exception table
 			*/
 			{
-				register int *yyxi = yyexca;
+				register const int *yyxi = yyexca;
 
 				while ( ( *yyxi != -1 ) ||
 					( yyxi[1] != yy_state ) )
@@ -986,27 +985,29 @@ case 12:
 				 &dims); /* grow as needed */
 		     dims[ndims].name = (char *) emalloc(strlen(yypvt[-0]->name)+1);
 		     (void) strcpy(dims[ndims].name, yypvt[-0]->name);
+		     /* name for use in generated Fortran and C variables */
+		     dims[ndims].lname = decodify(yypvt[-0]->name);
 		   } break;
 case 25:
-# line 166 "ncgen.y"
+# line 168 "ncgen.y"
 { type_code = NC_BYTE; } break;
 case 26:
-# line 167 "ncgen.y"
+# line 169 "ncgen.y"
 { type_code = NC_CHAR; } break;
 case 27:
-# line 168 "ncgen.y"
+# line 170 "ncgen.y"
 { type_code = NC_SHORT; } break;
 case 28:
-# line 169 "ncgen.y"
+# line 171 "ncgen.y"
 { type_code = NC_INT; } break;
 case 29:
-# line 170 "ncgen.y"
+# line 172 "ncgen.y"
 { type_code = NC_FLOAT; } break;
 case 30:
-# line 171 "ncgen.y"
+# line 173 "ncgen.y"
 { type_code = NC_DOUBLE; } break;
 case 33:
-# line 177 "ncgen.y"
+# line 179 "ncgen.y"
 {
 		    static struct vars dummyvar;
 
@@ -1031,6 +1032,8 @@ case 33:
 		    vars[nvars] = dummyvar; /* to make Purify happy */
 		    vars[nvars].name = (char *) emalloc(strlen(yypvt[-0]->name)+1);
 		    (void) strcpy(vars[nvars].name, yypvt[-0]->name);
+		    /* name for use in generated Fortran and C variables */
+		    vars[nvars].lname = decodify(yypvt[-0]->name);
 		    vars[nvars].type = type_code;
 		    /* set default fill value.  You can override this with
 		     * the variable attribute "_FillValue". */
@@ -1038,13 +1041,13 @@ case 33:
 		    vars[nvars].has_data = 0; /* has no data (yet) */
 		   } break;
 case 34:
-# line 208 "ncgen.y"
+# line 212 "ncgen.y"
 {
 		    vars[nvars].ndims = nvdims;
 		    nvars++;
 		   } break;
 case 40:
-# line 222 "ncgen.y"
+# line 226 "ncgen.y"
 {
 		    if (nvdims >= NC_MAX_VAR_DIMS) {
 		       derror("%s has too many dimensions",vars[nvars].name);
@@ -1065,7 +1068,7 @@ case 40:
                     nvdims++;
 		   } break;
 case 41:
-# line 243 "ncgen.y"
+# line 247 "ncgen.y"
 {
 		       valnum = 0;
 		       valtype = NC_UNSPECIFIED;
@@ -1073,14 +1076,14 @@ case 41:
 		       att_space = emalloc(MAX_NC_ATTSIZE);
 		       /* make all kinds of pointers point to it */
 		       char_valp = (char *) att_space;
-		       byte_valp = (char *) att_space;
+		       byte_valp = (signed char *) att_space;
 		       short_valp = (short *) att_space;
 		       int_valp = (int *) att_space;
 		       float_valp = (float *) att_space;
 		       double_valp = (double *) att_space;
 		   } break;
 case 42:
-# line 257 "ncgen.y"
+# line 261 "ncgen.y"
 {
 		       {	/* check if duplicate attribute for this var */
 			   int i;
@@ -1098,7 +1101,8 @@ case 42:
 		       /* shrink space down to what was really needed */
 		       att_space = erealloc(att_space, valnum*nctypesize(valtype));
 		       atts[natts].val = att_space;
-		       if (STREQ(atts[natts].name, _FillValue)) {
+		       if (STREQ(atts[natts].name, _FillValue) &&
+			   atts[natts].var != NC_GLOBAL) {
 			   nc_putfill(atts[natts].type,
 				       atts[natts].val,
 				       &vars[atts[natts].var].fill_value);
@@ -1110,12 +1114,12 @@ case 42:
 		       natts++;
 		   } break;
 case 44:
-# line 288 "ncgen.y"
+# line 293 "ncgen.y"
 {
-		    varnum = -1;  /* handle of "global" attribute */
+		    varnum = NC_GLOBAL;  /* handle of "global" attribute */
 		   } break;
 case 45:
-# line 294 "ncgen.y"
+# line 299 "ncgen.y"
 { if (yypvt[-0]->is_var == 1)
 		       varnum = yypvt[-0]->vnum;
 		    else {
@@ -1125,16 +1129,18 @@ case 45:
 		      }
 		   } break;
 case 46:
-# line 304 "ncgen.y"
+# line 309 "ncgen.y"
 {
 		       /* make sure atts array will hold attributes */
 		       grow_aarray(natts,  /* must hold natts+1 atts */
 				   &atts); /* grow as needed */
 		       atts[natts].name = (char *) emalloc(strlen(yypvt[-0]->name)+1);
 		       (void) strcpy(atts[natts].name,yypvt[-0]->name);
+		       /* name for use in generated Fortran and C variables */
+		       atts[natts].lname = decodify(yypvt[-0]->name);
 		   } break;
 case 49:
-# line 316 "ncgen.y"
+# line 323 "ncgen.y"
 {
 		    if (valtype == NC_UNSPECIFIED)
 		      valtype = atype_code;
@@ -1142,14 +1148,14 @@ case 49:
 		      derror("values for attribute must be all of same type");
 		   } break;
 case 50:
-# line 325 "ncgen.y"
+# line 332 "ncgen.y"
 {
 		       atype_code = NC_CHAR;
 		       *char_valp++ = char_val;
 		       valnum++;
 		   } break;
 case 51:
-# line 331 "ncgen.y"
+# line 338 "ncgen.y"
 {
 		       atype_code = NC_CHAR;
 		       {
@@ -1163,42 +1169,42 @@ case 51:
 		       }
 		   } break;
 case 52:
-# line 344 "ncgen.y"
+# line 351 "ncgen.y"
 {
 		       atype_code = NC_BYTE;
 		       *byte_valp++ = byte_val;
 		       valnum++;
 		   } break;
 case 53:
-# line 350 "ncgen.y"
+# line 357 "ncgen.y"
 {
 		       atype_code = NC_SHORT;
 		       *short_valp++ = short_val;
 		       valnum++;
 		   } break;
 case 54:
-# line 356 "ncgen.y"
+# line 363 "ncgen.y"
 {
 		       atype_code = NC_INT;
 		       *int_valp++ = int_val;
 		       valnum++;
 		   } break;
 case 55:
-# line 362 "ncgen.y"
+# line 369 "ncgen.y"
 {
 		       atype_code = NC_FLOAT;
 		       *float_valp++ = float_val;
 		       valnum++;
 		   } break;
 case 56:
-# line 368 "ncgen.y"
+# line 375 "ncgen.y"
 {
 		       atype_code = NC_DOUBLE;
 		       *double_valp++ = double_val;
 		       valnum++;
 		   } break;
 case 61:
-# line 383 "ncgen.y"
+# line 390 "ncgen.y"
 {
 		       valtype = vars[varnum].type; /* variable type */
 		       valnum = 0;	/* values accumulated for variable */
@@ -1232,7 +1238,7 @@ case 61:
 			   char_valp = (char *) rec_start;
 			   break;
 			 case NC_BYTE:
-			   byte_valp = (char *) rec_start;
+			   byte_valp = (signed char *) rec_start;
 			   break;
 			 case NC_SHORT:
 			   short_valp = (short *) rec_start;
@@ -1249,7 +1255,7 @@ case 61:
 		       }
 		 } break;
 case 62:
-# line 433 "ncgen.y"
+# line 440 "ncgen.y"
 {
 		       if (valnum < var_len) { /* leftovers */
 			   nc_fill(valtype,
@@ -1258,13 +1264,13 @@ case 62:
 				    vars[varnum].fill_value);
 		       }
 		       /* put out var_len values */
-		       vars[varnum].nrecs = var_len / rec_len;
+		       vars[varnum].nrecs = valnum / rec_len;
 		       if (derror_count == 0)
 			   put_variable(rec_start);
 		       free ((char *) rec_start);
 		 } break;
 case 65:
-# line 451 "ncgen.y"
+# line 458 "ncgen.y"
 {
 		       if(valnum >= var_len) {
 			   if (vars[varnum].dims[0] != rec_dim) { /* not recvar */
@@ -1280,7 +1286,7 @@ case 65:
 			       rec_start = erealloc(rec_start, var_len*var_size);
 			       rec_cur = (char *)rec_start + rec_inc;
 			       char_valp = (char *) rec_cur;
-			       byte_valp = (char *) rec_cur;
+			       byte_valp = (signed char *) rec_cur;
 			       short_valp = (short *) rec_cur;
 			       int_valp = (int *) rec_cur;
 			       float_valp = (float *) rec_cur;
@@ -1290,7 +1296,7 @@ case 65:
 		       not_a_string = 1;
                    } break;
 case 66:
-# line 476 "ncgen.y"
+# line 483 "ncgen.y"
 {
 		       if (not_a_string) {
 			   switch (valtype) {
@@ -1316,7 +1322,7 @@ case 66:
 		       }
 		   } break;
 case 67:
-# line 503 "ncgen.y"
+# line 510 "ncgen.y"
 {
 		       atype_code = NC_CHAR;
 		       switch (valtype) {
@@ -1342,7 +1348,7 @@ case 67:
 		       valnum++;
 		   } break;
 case 68:
-# line 528 "ncgen.y"
+# line 535 "ncgen.y"
 {
 		       not_a_string = 0;
 		       atype_code = NC_CHAR;
@@ -1377,9 +1383,12 @@ case 68:
 					   char_valp[i] = '\0';
 				       if (sl < len)
 					   sl = len;
+				       valnum += sl;
+				       char_valp += sl;
+				   } else { /* scalar or 1D strings */
+				       valnum += len;
+				       char_valp += len;
 				   }
-				   valnum += len;
-				   char_valp += len;
 				   rec_cur = (void *) char_valp;
 			       }
 			       break;
@@ -1395,7 +1404,7 @@ case 68:
 		       }
 		   } break;
 case 69:
-# line 580 "ncgen.y"
+# line 590 "ncgen.y"
 {
 		       atype_code = NC_BYTE;
 		       switch (valtype) {
@@ -1421,7 +1430,7 @@ case 69:
 		       valnum++;
 		   } break;
 case 70:
-# line 605 "ncgen.y"
+# line 615 "ncgen.y"
 {
 		       atype_code = NC_SHORT;
 		       switch (valtype) {
@@ -1447,7 +1456,7 @@ case 70:
 		       valnum++;
 		   } break;
 case 71:
-# line 630 "ncgen.y"
+# line 640 "ncgen.y"
 {
 		       atype_code = NC_INT;
 		       switch (valtype) {
@@ -1473,7 +1482,7 @@ case 71:
 		       valnum++;
 		   } break;
 case 72:
-# line 655 "ncgen.y"
+# line 665 "ncgen.y"
 {
 		       atype_code = NC_FLOAT;
 		       switch (valtype) {
@@ -1499,7 +1508,7 @@ case 72:
 		       valnum++;
 		   } break;
 case 73:
-# line 680 "ncgen.y"
+# line 690 "ncgen.y"
 {
 		       atype_code = NC_DOUBLE;
 		       switch (valtype) {
@@ -1528,7 +1537,7 @@ case 73:
 		       valnum++;
 		   } break;
 case 74:
-# line 708 "ncgen.y"
+# line 718 "ncgen.y"
 {
 		       /* store fill_value */
 		       switch (valtype) {
@@ -1559,7 +1568,7 @@ case 74:
 		       }
 		       valnum++;
 		   } break;
-# line	532 "/usr/ccs/bin/yaccpar"
+# line	531 "/usr/ccs/bin/yaccpar"
 	}
 	goto yystack;		/* reset registers in driver code */
 }
