@@ -1,5 +1,5 @@
 /*
- * $Id: fortlib.c,v 1.4 1997/06/11 16:20:17 steve Exp $
+ * $Id: fortlib.c,v 1.5 1997/12/10 17:06:15 steve Exp $
  *
  * This file contains support functions for FORTRAN code.  For example,
  * under HP-UX A.09.05, the U77 library doesn't contain the exit()
@@ -9,10 +9,10 @@
 
 #include <stdlib.h>
 
-#include "../fortran/cfortran.h"
+#include "../fortran/ncfortran.h"
 
 
-FCALLSCSUB1(exit, UDEXIT, udexit, INT)
+FCALLSCSUB1(exit, UDEXIT, udexit, FINT2CINT)
 
 
 FCALLSCSUB0(abort, UDABORT, udabort)
@@ -32,7 +32,7 @@ myrand(int iflag)
      */
     return (rand() % 32768) / 32767.0;
 }
-FCALLSCFUN1(DOUBLE, myrand, UDRAND, udrand, INT)
+FCALLSCFUN1(DOUBLE, myrand, UDRAND, udrand, FINT2CINT)
 
 
 static int
@@ -45,7 +45,7 @@ myshift(int value, int amount)
 	value <<= amount;
     return value;
 }
-FCALLSCFUN2(INT, myshift, UDSHIFT, udshift, INT, INT)
+FCALLSCFUN2(NF_INT, myshift, UDSHIFT, udshift, FINT2CINT, FINT2CINT)
 
 #include <signal.h>
 static void
@@ -54,4 +54,4 @@ nc_ignorefpe(int doit)
 	if(doit)
 		(void) signal(SIGFPE, SIG_IGN);
 }
-FCALLSCSUB1(nc_ignorefpe, IGNOREFPE, ignorefpe, INT)
+FCALLSCSUB1(nc_ignorefpe, IGNOREFPE, ignorefpe, FINT2CINT)
